searchData={"items":[{"type":"module","title":"Aja","doc":"Convenience macros to work with Aja's data structures.\n\nUse `import Aja` to import everything, or import only the macros you need.","ref":"Aja.html"},{"type":"function","title":"Aja.+++/2","doc":"Convenience operator to concatenate an enumerable `right` to a vector `left`.\n\n`left` has to be an `Aja.Vector`, `right` can be any `Enumerable`.\n\nIt is just an alias for `Aja.Vector.concat/2`.\n\nOnly available on Elixir versions >= 1.11.","ref":"Aja.html#+++/2"},{"type":"function","title":"Examples - Aja.+++/2","doc":"iex> import Aja\n    iex> vec(5..1) +++ vec([:boom, nil])\n    vec([5, 4, 3, 2, 1, :boom, nil])\n    iex> vec(5..1) +++ 0..3\n    vec([5, 4, 3, 2, 1, 0, 1, 2, 3])","ref":"Aja.html#+++/2-examples"},{"type":"macro","title":"Aja.ord/1","doc":"Convenience macro to create or pattern match on `Aja.OrdMap`s.\n\nUse `import Aja` to use it, or `import Aja, only: [ord: 1]`.","ref":"Aja.html#ord/1"},{"type":"macro","title":"Creation examples - Aja.ord/1","doc":"iex> ord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\n    ord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\n    iex> ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.html#ord/1-creation-examples"},{"type":"macro","title":"Pattern matching examples - Aja.ord/1","doc":"iex> ord(%{b: bat}) = ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"}); bat\n    \"Bat\"","ref":"Aja.html#ord/1-pattern-matching-examples"},{"type":"macro","title":"Replace existing keys examples - Aja.ord/1","doc":"iex> ordered = ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})\n    iex> ord(%{ordered | b: \"Buffalo\"})\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> ord(%{ordered | z: \"Zebra\"})\n    ** (KeyError) key :z not found in: ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.html#ord/1-replace-existing-keys-examples"},{"type":"macro","title":"Aja.ord_size/1","doc":"Returns the size of an `ord_map`.\n\nIt is implemented as a macro so that it can be used in guards.\n\nWhen used outside of a guard, it will just be replaced by a call to `Aja.OrdMap.size/1`.\n\nWhen used in guards, it will fail if called on something else than an `Aja.OrdMap`.\nIt is recommended to verify the type first.\n\nRuns in constant time.","ref":"Aja.html#ord_size/1"},{"type":"macro","title":"Examples - Aja.ord_size/1","doc":"iex> import Aja\n    iex> ord_map = Aja.OrdMap.new(a: 1, b: 2, c: 3)\n    iex> match?(v when ord_size(v) > 5, ord_map)\n    false\n    iex> match?(v when ord_size(v) < 5, ord_map)\n    true\n    iex> ord_size(ord_map)\n    3","ref":"Aja.html#ord_size/1-examples"},{"type":"macro","title":"Aja.sigil_i/2","doc":"A sigil to build [IO data](https://hexdocs.pm/elixir/IO.html#module-io-data) and avoid string concatenation.\n\nUse `import Aja` to use it, or `import Aja, only: [sigil_i: 2]`.\n\nThis sigil provides a faster version of string interpolation which:\n- will build a list with all chunks instead of concatenating them as a string\n- uses `Aja.IO.to_iodata/1` on interpolated values instead of `to_string/1`, which:\n  * will keep lists untouched, without any validation or transformation\n  * will cast anything else using `to_string/1`\n\nWorks with both [IO data](https://hexdocs.pm/elixir/IO.html#module-io-data) and\n[Chardata](https://hexdocs.pm/elixir/IO.html?#module-chardata).\nSee their respective documentation for more information.","ref":"Aja.html#sigil_i/2"},{"type":"macro","title":"Examples - Aja.sigil_i/2","doc":"iex> ~i\"atom: #{:foo}, charlist: #{'abc'}, number: #{12 + 2.35}\\n\"\n    [\"atom: \", \"foo\", \", charlist: \", 'abc', \", number: \", \"14.35\", 10]\n    iex> ~i\"abc#{['def' | \"ghi\"]}\"\n    [\"abc\", ['def' | \"ghi\"]]\n    iex> ~i\"Giorno Giovanna\"\n    \"Giorno Giovanna\"\n\n  IO data can often be used as is without ever generating the corresponding string.\n  If needed however, IO data can be cast as a string using `IO.iodata_to_binary/1`,\n  and chardata using `List.to_string/1`. In most cases, both should be the same:\n\n    iex> IO.iodata_to_binary(~i\"atom: #{:foo}, charlist: #{'abc'}, number: #{12 + 2.35}\\n\")\n    \"atom: foo, charlist: abc, number: 14.35\\n\"\n    iex> List.to_string(~i\"abc#{['def' | \"ghi\"]}\")\n    \"abcdefghi\"\n\n  Those are the exact same values returned by a regular string interpolation, without\n  the `~i` sigil:\n\n    iex> \"atom: #{:foo}, charlist: #{'abc'}, number: #{12 + 2.35}\\n\"\n    \"atom: foo, charlist: abc, number: 14.35\\n\"\n    iex> \"abc#{['def' | \"ghi\"]}\"\n    \"abcdefghi\"","ref":"Aja.html#sigil_i/2-examples"},{"type":"macro","title":"Aja.vec/1","doc":"Convenience macro to create or pattern match on `Aja.Vector`s.","ref":"Aja.html#vec/1"},{"type":"macro","title":"Examples - Aja.vec/1","doc":"iex> import Aja\n    iex> vec([1, 2, 3])\n    vec([1, 2, 3])\n    iex> vec(first ||| last) = Aja.Vector.new(0..99_999); {first, last}\n    {0, 99999}\n    iex> vec([1, 2, var, _, _, _]) = Aja.Vector.new(1..6); var\n    3\n    iex> vec([_, _, _]) = Aja.Vector.new(1..6)\n    ** (MatchError) no match of right hand side value: vec([1, 2, 3, 4, 5, 6])\n\nIt also supports ranges with **constant** values:\n\n    iex> vec(0..4) = Aja.Vector.new(0..4)\n    vec([0, 1, 2, 3, 4])\n\nVariable lists or dynamic ranges cannot be passed:\n\n    vec(my_list)  # invalid\n    vec(1..n)  # invalid","ref":"Aja.html#vec/1-examples"},{"type":"macro","title":"Explanation - Aja.vec/1","doc":"The `vec/1` macro generates the AST at compile time instead of building the vector\nat runtime. This can speedup the instanciation of vectors of known size.","ref":"Aja.html#vec/1-explanation"},{"type":"macro","title":"Aja.vec_size/1","doc":"Returns the size of a `vector`.\n\nIt is implemented as a macro so that it can be used in guards.\n\nWhen used outside of a guard, it will just be replaced by a call to `Aja.Vector.size/1`.\n\nWhen used in guards, it will fail if called on something else than an `Aja.Vector`.\nIt is recommended to verify the type first.\n\nRuns in constant time.","ref":"Aja.html#vec_size/1"},{"type":"macro","title":"Examples - Aja.vec_size/1","doc":"iex> import Aja\n    iex> match?(v when vec_size(v) > 20, Aja.Vector.new(1..10))\n    false\n    iex> match?(v when vec_size(v) < 5, Aja.Vector.new([1, 2, 3]))\n    true\n    iex> vec_size(Aja.Vector.new([1, 2, 3]))\n    3","ref":"Aja.html#vec_size/1-examples"},{"type":"module","title":"Aja.Enum","doc":"Drop-in replacement for the `Enum` module, optimized to work with Aja's data structures such as `Aja.Vector`.\n\nIt currently only covers a subset of `Enum`, but `Aja.Enum` aims to completely mirror the API of `Enum`,\nand should behave exactly the same for any type of `Enumerable`.\nThe only expected difference should be a significant increase in performance for Aja structures.","ref":"Aja.Enum.html"},{"type":"module","title":"Rationale - Aja.Enum","doc":"Structures such as `Aja.Vector` or `Aja.OrdMap` are implementing the `Enumerable` protocol, which means they can be\nused directly with the `Enum` module. The `Enumerable` protocol however comes with its overhead and is strongly\nlimited in terms of performance.\n\nOn the other hand, `Aja.Enum` provides hand-crafted highly-optimized functions that fully take advantage of\nimmutable vectors. The speedup can easily reach more than a factor 10 compared to `Enum` used on non-list\nstructures, and sometimes even be noticeably faster than `Enum` used over lists.\n\nOne of the main reasons to adopt a specific data structure is the performance.\nUsing vectors with `Enum` would defeat the purpose, hence the introduction of `Aja.Enum`.\n\n    iex> vector = Aja.Vector.new(1..10000)\n    iex> Enum.sum(vector)      # slow\n    50005000\n    iex> Aja.Enum.sum(vector)  # same result, much faster\n    50005000","ref":"Aja.Enum.html#module-rationale"},{"type":"function","title":"Aja.Enum.all?/1","doc":"Returns `true` if  all elements in `enumerable` are truthy.\n\nWhen an element has a falsy value (`false` or `nil`) iteration stops immediately\nand `false` is returned. In all other cases `true` is returned.","ref":"Aja.Enum.html#all?/1"},{"type":"function","title":"Examples - Aja.Enum.all?/1","doc":"iex> Aja.Enum.all?([1, 2, 3])\n    true\n\n    iex> Aja.Enum.all?([1, nil, 3])\n    false\n\n    iex> Aja.Enum.all?([])\n    true","ref":"Aja.Enum.html#all?/1-examples"},{"type":"function","title":"Aja.Enum.all?/2","doc":"Returns `true` if `fun.(element)` is truthy for all elements in `enumerable`.\n\nIterates over `enumerable` and invokes `fun` on each element. If `fun` ever\nreturns a falsy value (`false` or `nil`), iteration stops immediately and\n`false` is returned. Otherwise, `true` is returned.","ref":"Aja.Enum.html#all?/2"},{"type":"function","title":"Examples - Aja.Enum.all?/2","doc":"iex> Aja.Enum.all?([2, 4, 6], fn x -> rem(x, 2) == 0 end)\n    true\n\n    iex> Aja.Enum.all?([2, 3, 4], fn x -> rem(x, 2) == 0 end)\n    false\n\n    iex> Aja.Enum.all?([], fn _ -> nil end)\n    true","ref":"Aja.Enum.html#all?/2-examples"},{"type":"function","title":"Aja.Enum.any?/1","doc":"Returns `true` if at least one element in `enumerable` is truthy.\n\nWhen an element has a truthy value (neither `false` nor `nil`) iteration stops\nimmediately and `true` is returned. In all other cases `false` is returned.","ref":"Aja.Enum.html#any?/1"},{"type":"function","title":"Examples - Aja.Enum.any?/1","doc":"iex> Aja.Enum.any?([false, false, false])\n    false\n\n    iex> Aja.Enum.any?([false, true, false])\n    true\n\n    iex> Aja.Enum.any?([])\n    false","ref":"Aja.Enum.html#any?/1-examples"},{"type":"function","title":"Aja.Enum.any?/2","doc":"Returns `true` if `fun.(element)` is truthy for at least one element in `enumerable`.\n\nIterates over the `enumerable` and invokes `fun` on each element. When an invocation\nof `fun` returns a truthy value (neither `false` nor `nil`) iteration stops\nimmediately and `true` is returned. In all other cases `false` is returned.","ref":"Aja.Enum.html#any?/2"},{"type":"function","title":"Examples - Aja.Enum.any?/2","doc":"iex> Aja.Enum.any?([2, 4, 6], fn x -> rem(x, 2) == 1 end)\n    false\n\n    iex> Aja.Enum.any?([2, 3, 4], fn x -> rem(x, 2) == 1 end)\n    true\n\n    iex> Aja.Enum.any?([], fn x -> x > 0 end)\n    false","ref":"Aja.Enum.html#any?/2-examples"},{"type":"function","title":"Aja.Enum.at/3","doc":"Finds the element at the given `index` (zero-based).\n\nMirrors `Enum.at/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#at/3"},{"type":"function","title":"Aja.Enum.concat/1","doc":"Given an enumerable of enumerables, concatenates the `enumerables` into\na single list.\n\nMirrors `Enum.concat/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#concat/1"},{"type":"function","title":"Aja.Enum.concat/2","doc":"Concatenates the enumerable on the `right` with the enumerable on the `left`.\n\nMirrors `Enum.concat/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#concat/2"},{"type":"function","title":"Aja.Enum.count/1","doc":"Returns the size of the `enumerable`.\n\nMirrors `Enum.count/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#count/1"},{"type":"function","title":"Aja.Enum.count/2","doc":"Returns the count of elements in the `enumerable` for which `fun` returns\na truthy value.\n\nMirrors `Enum.count/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#count/2"},{"type":"function","title":"Aja.Enum.dedup/1","doc":"Enumerates the `enumerable`, returning a list where all consecutive\nduplicated elements are collapsed to a single element.\n\nMirrors `Enum.dedup/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#dedup/1"},{"type":"function","title":"Aja.Enum.dedup_by/2","doc":"Enumerates the `enumerable`, returning a list where all consecutive\nduplicated elements are collapsed to a single element.\n\nMirrors `Enum.dedup_by/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#dedup_by/2"},{"type":"function","title":"Aja.Enum.drop/2","doc":"Drops the `amount` of elements from the `enumerable`.\n\nMirrors `Enum.drop/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#drop/2"},{"type":"function","title":"Aja.Enum.drop_while/2","doc":"Drops elements at the beginning of the `enumerable` while `fun` returns a truthy value.\n\nMirrors `Enum.drop_while/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#drop_while/2"},{"type":"function","title":"Aja.Enum.each/2","doc":"Invokes the given `fun` for each element in the `enumerable`.\n\nMirrors `Enum.each/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#each/2"},{"type":"function","title":"Aja.Enum.empty?/1","doc":"Returns `true` if `enumerable` is empty, otherwise `false`.\n\nMirrors `Enum.empty?/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#empty?/1"},{"type":"function","title":"Aja.Enum.fetch/2","doc":"Finds the element at the given `index` (zero-based).\n\nMirrors `Enum.fetch/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#fetch/2"},{"type":"function","title":"Aja.Enum.fetch!/2","doc":"Finds the element at the given `index` (zero-based).\n\nMirrors `Enum.fetch!/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#fetch!/2"},{"type":"function","title":"Aja.Enum.filter/2","doc":"Filters the `enumerable`, i.e. returns only those elements\nfor which `fun` returns a truthy value.\n\nMirrors `Enum.filter/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#filter/2"},{"type":"function","title":"Aja.Enum.find/3","doc":"Returns the first element for which `fun` returns a truthy value.\nIf no such element is found, returns `default`.\n\nMirrors `Enum.find/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#find/3"},{"type":"function","title":"Aja.Enum.find_index/2","doc":"Similar to `find/3`, but returns the index (zero-based)\nof the element instead of the element itself.\n\nMirrors `Enum.find_index/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#find_index/2"},{"type":"function","title":"Aja.Enum.find_value/3","doc":"Similar to `find/3`, but returns the value of the function\ninvocation instead of the element itself.\n\nMirrors `Enum.find_value/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#find_value/3"},{"type":"function","title":"Aja.Enum.flat_map/2","doc":"Maps the given `fun` over `enumerable` and flattens the result.\n\nMirrors `Enum.flat_map/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#flat_map/2"},{"type":"function","title":"Aja.Enum.frequencies/1","doc":"Returns a map with keys as unique elements of `enumerable` and values\nas the count of every element.\n\nMirrors `Enum.frequencies/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#frequencies/1"},{"type":"function","title":"Aja.Enum.frequencies_by/2","doc":"Returns a map with keys as unique elements given by `key_fun` and values\nas the count of every element.\n\nMirrors `Enum.frequencies_by/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#frequencies_by/2"},{"type":"function","title":"Aja.Enum.group_by/3","doc":"Splits the `enumerable` into groups based on `key_fun`.\n\nMirrors `Enum.group_by/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#group_by/3"},{"type":"function","title":"Aja.Enum.intersperse/2","doc":"Intersperses `separator` between each element of the given `enumerable`.\n\nMirrors `Enum.intersperse/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#intersperse/2"},{"type":"function","title":"Aja.Enum.into/2","doc":"Inserts the given `enumerable` into a `collectable`.\n\nMirrors `Enum.into/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#into/2"},{"type":"function","title":"Aja.Enum.into/3","doc":"Inserts the given `enumerable` into a `collectable` according to the `transform` function.\n\nMirrors `Enum.into/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#into/3"},{"type":"function","title":"Aja.Enum.join/2","doc":"Joins the given `enumerable` into a string using `joiner` as a\nseparator.\n\nMirrors `Enum.join/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#join/2"},{"type":"function","title":"Aja.Enum.map/2","doc":"Returns a list where each element is the result of invoking\n`fun` on each corresponding element of `enumerable`.\n\nMirrors `Enum.map/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#map/2"},{"type":"function","title":"Aja.Enum.map_intersperse/3","doc":"Maps and intersperses the given `enumerable` in one pass.\n\nMirrors `Enum.map_intersperse/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#map_intersperse/3"},{"type":"function","title":"Aja.Enum.map_join/3","doc":"Maps and joins the given `enumerable` in one pass.\n\nMirrors `Enum.map_join/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#map_join/3"},{"type":"function","title":"Aja.Enum.map_reduce/3","doc":"Invokes the given function to each element in the `enumerable` to reduce\nit to a single element, while keeping an accumulator.\n\nMirrors `Enum.map_reduce/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#map_reduce/3"},{"type":"function","title":"Aja.Enum.max/3","doc":"Returns the maximal element in the `enumerable` according\nto Erlang's term ordering.\n\nMirrors `Enum.max/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#max/3"},{"type":"function","title":"Aja.Enum.max_by/4","doc":"Returns the maximal element in the `enumerable` as calculated\nby the given `fun`.\n\nMirrors `Enum.max_by/4` with higher performance for Aja structures.","ref":"Aja.Enum.html#max_by/4"},{"type":"function","title":"Aja.Enum.member?/2","doc":"Checks if `element` exists within the `enumerable`.\n\nJust an alias for `Enum.member?/2`, does not improve performance.","ref":"Aja.Enum.html#member?/2"},{"type":"function","title":"Aja.Enum.min/3","doc":"Returns the minimal element in the `enumerable` according\nto Erlang's term ordering.\n\nMirrors `Enum.min/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#min/3"},{"type":"function","title":"Aja.Enum.min_by/4","doc":"Returns the minimal element in the `enumerable` as calculated\nby the given `fun`.\n\nMirrors `Enum.min_by/4` with higher performance for Aja structures.","ref":"Aja.Enum.html#min_by/4"},{"type":"function","title":"Aja.Enum.product/1","doc":"Returns the product of all elements in the `enumerable`.\n\nMirrors `Enum.product/1`.\n\nRaises `ArithmeticError` if `enumerable` contains a non-numeric value.","ref":"Aja.Enum.html#product/1"},{"type":"function","title":"Examples - Aja.Enum.product/1","doc":"iex> 1..5 |> Aja.Enum.product()\n    120\n    iex> [] |> Aja.Enum.product()\n    1","ref":"Aja.Enum.html#product/1-examples"},{"type":"function","title":"Aja.Enum.random/1","doc":"Returns a random element of an `enumerable`.\n\nMirrors `Enum.random/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#random/1"},{"type":"function","title":"Aja.Enum.reduce/2","doc":"Invokes `fun` for each element in the `enumerable` with the\naccumulator.\n\nMirrors `Enum.reduce/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#reduce/2"},{"type":"function","title":"Aja.Enum.reduce/3","doc":"Invokes `fun` for each element in the `enumerable` with the accumulator.\n\nMirrors `Enum.reduce/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#reduce/3"},{"type":"function","title":"Aja.Enum.reject/2","doc":"Returns a list of elements in `enumerable` excluding those for which the function `fun` returns\na truthy value.\n\nMirrors `Enum.reject/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#reject/2"},{"type":"function","title":"Aja.Enum.reverse/1","doc":"Returns a list of elements in `enumerable` in reverse order.\n\nMirrors `Enum.reverse/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#reverse/1"},{"type":"function","title":"Aja.Enum.reverse/2","doc":"Reverses the elements in `enumerable`, concatenates the `tail`,\nand returns it as a list.\n\nMirrors `Enum.reverse/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#reverse/2"},{"type":"function","title":"Aja.Enum.scan/2","doc":"Applies the given function to each element in the `enumerable`,\nstoring the result in a list and passing it as the accumulator\nfor the next computation. Uses the first element in the `enumerable`\nas the starting value.\n\nMirrors `Enum.scan/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#scan/2"},{"type":"function","title":"Aja.Enum.scan/3","doc":"Applies the given function to each element in the `enumerable`,\nstoring the result in a list and passing it as the accumulator\nfor the next computation. Uses the given `acc` as the starting value.\n\nMirrors `Enum.scan/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#scan/3"},{"type":"function","title":"Aja.Enum.shuffle/1","doc":"Returns a list with the elements of `enumerable` shuffled.\n\nMirrors `Enum.shuffle/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#shuffle/1"},{"type":"function","title":"Aja.Enum.slice/2","doc":"Returns a subset list of the given `enumerable` by `index_range`.\n\nMirrors `Enum.slice/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#slice/2"},{"type":"function","title":"Aja.Enum.slice/3","doc":"Returns a subset list of the given `enumerable`, from `start_index` (zero-based)\nwith `amount` number of elements if available.\n\nMirrors `Enum.slice/3`.","ref":"Aja.Enum.html#slice/3"},{"type":"function","title":"Aja.Enum.sort/1","doc":"Sorts the `enumerable` according to Erlang's term ordering.\n\nMirrors `Enum.sort/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#sort/1"},{"type":"function","title":"Aja.Enum.sort/2","doc":"Sorts the `enumerable` by the given function.\n\nMirrors `Enum.sort/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#sort/2"},{"type":"function","title":"Aja.Enum.sort_by/3","doc":"Sorts the mapped results of the `enumerable` according to the provided `sorter`\nfunction.\n\nMirrors `Enum.sort_by/3` with higher performance for Aja structures.","ref":"Aja.Enum.html#sort_by/3"},{"type":"function","title":"Aja.Enum.split/2","doc":"Splits the `enumerable` into two enumerables, leaving `count` elements in the first one.\n\nMirrors `Enum.split/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#split/2"},{"type":"function","title":"Aja.Enum.split_while/2","doc":"Splits `enumerable` in two at the position of the element for which `fun` returns a falsy value\n(`false` or `nil`) for the first time.\n\nMirrors `Enum.split_while/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#split_while/2"},{"type":"function","title":"Aja.Enum.split_with/2","doc":"Splits the `enumerable` in two lists according to the given function `fun`.\n\nMirrors `Enum.split_with/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#split_with/2"},{"type":"function","title":"Aja.Enum.sum/1","doc":"Returns the sum of all elements.\n\nMirrors `Enum.sum/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#sum/1"},{"type":"function","title":"Aja.Enum.take/2","doc":"Takes an `amount` of elements from the beginning or the end of the `enumerable`.\n\nMirrors `Enum.take/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#take/2"},{"type":"function","title":"Aja.Enum.take_random/2","doc":"Takes `count` random elements from `enumerable`.\n\nMirrors `Enum.take_random/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#take_random/2"},{"type":"function","title":"Aja.Enum.take_while/2","doc":"Takes the elements from the beginning of the `enumerable` while `fun` returns a truthy value.\n\nMirrors `Enum.take_while/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#take_while/2"},{"type":"function","title":"Aja.Enum.to_list/1","doc":"Converts `enumerable` to a list.\n\nMirrors `Enum.to_list/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#to_list/1"},{"type":"function","title":"Aja.Enum.uniq/1","doc":"Enumerates the `enumerable`, removing all duplicated elements.\n\nMirrors `Enum.uniq/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#uniq/1"},{"type":"function","title":"Aja.Enum.uniq_by/2","doc":"Enumerates the `enumerable`, by removing the elements for which\nfunction `fun` returned duplicate elements.\n\nMirrors `Enum.uniq_by/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#uniq_by/2"},{"type":"function","title":"Aja.Enum.unzip/1","doc":"Opposite of `zip/2`. Extracts two-element tuples from the given `enumerable`\nand groups them together.\n\nMirrors `Enum.unzip/1` with higher performance for Aja structures.","ref":"Aja.Enum.html#unzip/1"},{"type":"function","title":"Aja.Enum.with_index/2","doc":"Returns a list with with each element of `enumerable` wrapped in a tuple alongside its index.\n\nMirrors `Enum.with_index/2`: may receive a function or an integer offset.\n\nIf an integer `offset` is given, it will index from the given `offset` instead of from zero.\n\nIf a `function` is given, it will index by invoking the function for each\nelement and index (zero-based) of the `enumerable`.","ref":"Aja.Enum.html#with_index/2"},{"type":"function","title":"Examples - Aja.Enum.with_index/2","doc":"iex> Aja.Enum.with_index([:a, :b, :c])\n    [a: 0, b: 1, c: 2]\n\n    iex> Aja.Enum.with_index([:a, :b, :c], 3)\n    [a: 3, b: 4, c: 5]\n\n    iex> Aja.Enum.with_index([:a, :b, :c], fn element, index -> {index, element} end)\n    [{0, :a}, {1, :b}, {2, :c}]","ref":"Aja.Enum.html#with_index/2-examples"},{"type":"function","title":"Aja.Enum.zip/2","doc":"Zips corresponding elements from two enumerables into one list of tuples.\n\nMirrors `Enum.zip/2` with higher performance for Aja structures.","ref":"Aja.Enum.html#zip/2"},{"type":"type","title":"Aja.Enum.index/0","doc":"","ref":"Aja.Enum.html#t:index/0"},{"type":"type","title":"Aja.Enum.t/1","doc":"","ref":"Aja.Enum.html#t:t/1"},{"type":"type","title":"Aja.Enum.value/0","doc":"","ref":"Aja.Enum.html#t:value/0"},{"type":"module","title":"Aja.IO","doc":"Some extra helper functions for working with IO data,\nthat are not in the core `IO` module.","ref":"Aja.IO.html"},{"type":"function","title":"Aja.IO.iodata_empty?/1","doc":"Checks if IO data is empty in \"constant\" time.\n\nShould only need to loop until it finds one character or binary to stop,\nunlike `IO.iodata_length(iodata) == 0` which needs to perform the complete loop\nto compute the length first.","ref":"Aja.IO.html#iodata_empty?/1"},{"type":"function","title":"Examples - Aja.IO.iodata_empty?/1","doc":"iex> Aja.IO.iodata_empty?([\"\", []])\n    true\n    iex> Aja.IO.iodata_empty?('a')\n    false\n    iex> Aja.IO.iodata_empty?([\"a\"])\n    false\n    iex> Aja.IO.iodata_empty?([\"\", [], [\"\" | \"c\"]])\n    false","ref":"Aja.IO.html#iodata_empty?/1-examples"},{"type":"function","title":"Rationale - Aja.IO.iodata_empty?/1","doc":"Even if `IO.iodata_length/1` is a very efficient BIF implemented in C, it has a linear\nalgorithmic complexity and can become slow if invoked on an IO list with many elements.\n\nThis is not a far-fetched scenario, and a production use case can easily include\n\"big\" IO-lists with:\n- JSON encoding to IO-data of long lists / nested objects\n- loops within HTML templates","ref":"Aja.IO.html#iodata_empty?/1-rationale"},{"type":"function","title":"Aja.IO.to_iodata/1","doc":"Converts the argument to IO data according to the `String.Chars` protocol.\n\nLeaves lists untouched without any validation, calls `to_string/1` on everything else.\n\nThis is the function invoked in string interpolations within the [i sigil](`Aja.sigil_i/2`).\n\nWorks with both [IO data](https://hexdocs.pm/elixir/IO.html#module-io-data) and\n[Chardata](https://hexdocs.pm/elixir/IO.html?#module-chardata),\ndepending on the type of the `data` parameter.","ref":"Aja.IO.html#to_iodata/1"},{"type":"function","title":"Examples - Aja.IO.to_iodata/1","doc":"iex> Aja.IO.to_iodata(:foo)\n    \"foo\"\n    iex> Aja.IO.to_iodata(99)\n    \"99\"\n    iex> Aja.IO.to_iodata([\"abc\", 'def' | \"ghi\"])\n    [\"abc\", 'def' | \"ghi\"]","ref":"Aja.IO.html#to_iodata/1-examples"},{"type":"module","title":"Aja.Integer","doc":"Some extra helper functions for working with integers,\nthat are not in the core `Integer` module.","ref":"Aja.Integer.html"},{"type":"function","title":"Aja.Integer.decimal_format/2","doc":"Format integers for humans, with thousand separators.","ref":"Aja.Integer.html#decimal_format/2"},{"type":"function","title":"Examples - Aja.Integer.decimal_format/2","doc":"iex> Aja.Integer.decimal_format(1_234_567)\n    \"1,234,567\"\n    iex> Aja.Integer.decimal_format(-123)\n    \"-123\"\n    iex> Aja.Integer.decimal_format(-1_234, separator: ?_)\n    \"-1_234\"","ref":"Aja.Integer.html#decimal_format/2-examples"},{"type":"function","title":"Aja.Integer.div_rem/2","doc":"Returns both the result of `div/2` and `rem/2` at once.","ref":"Aja.Integer.html#div_rem/2"},{"type":"function","title":"Examples - Aja.Integer.div_rem/2","doc":"iex> Aja.Integer.div_rem(7, 3)\n    {2, 1}\n    iex> Aja.Integer.div_rem(-99, 2)\n    {-49, -1}\n    iex> Aja.Integer.div_rem(100, 0)\n    ** (ArithmeticError) bad argument in arithmetic expression","ref":"Aja.Integer.html#div_rem/2-examples"},{"type":"module","title":"Aja.List","doc":"Some extra helper functions for working with lists,\nthat are not in the core `List` module.","ref":"Aja.List.html"},{"type":"function","title":"Aja.List.prepend/2","doc":"Prepends an element to a list, equivalent of `[elem | list]` that can be used in a pipe.","ref":"Aja.List.html#prepend/2"},{"type":"function","title":"Examples - Aja.List.prepend/2","doc":"iex> [2, 3, 5, 8] |> Aja.List.prepend(1)\n    [1, 2, 3, 5, 8]","ref":"Aja.List.html#prepend/2-examples"},{"type":"function","title":"Aja.List.repeat/2","doc":"Populates a list of size `n` by calling `generator_fun` repeatedly.","ref":"Aja.List.html#repeat/2"},{"type":"function","title":"Examples - Aja.List.repeat/2","doc":"# Although not necessary, let's seed the random algorithm\n    iex> :rand.seed(:exrop, {1, 2, 3})\n    iex> Aja.List.repeat(&:rand.uniform/0, 3)\n    [0.7498295129076106, 0.06161655489244533, 0.7924073127680873]\n\n    # It is basically just syntactic sugar for the following:\n    iex> Stream.repeatedly(&:rand.uniform/0) |> Enum.take(3)","ref":"Aja.List.html#repeat/2-examples"},{"type":"function","title":"Rationale - Aja.List.repeat/2","doc":"- It offers has a consistent API with `Stream.repeatedly/1` and `List.duplicate/2`\n- It provides a less verbose way of writing one of the most common uses of `Stream.repeatedly/1`\n- (before Elixir 1.12) It removes the temptation to write the following, which is more concise but is technically incorrect:\n\n      iex> incorrect = fn n -> for _i <- 1..n, do: :rand.uniform() end\n      iex> incorrect.(0) |> length()\n      2\n      iex> Enum.to_list(1..0)  # <- because of this\n      [1, 0]\n\n  Elixir 1.12 solved this problem by introducing a step, `1..n//1`.\n\n- It is more efficient","ref":"Aja.List.html#repeat/2-rationale"},{"type":"module","title":"Aja.OrdMap","doc":"A map preserving key insertion order, with efficient lookups, updates and enumeration.\n\nIt works like regular maps, except that the insertion order is preserved:\n\n    iex> %{\"one\" => 1, \"two\" => 2, \"three\" => 3}\n    %{\"one\" => 1, \"three\" => 3, \"two\" => 2}\n    iex> Aja.OrdMap.new([{\"one\", 1}, {\"two\", 2}, {\"three\", 3}])\n    ord(%{\"one\" => 1, \"two\" => 2, \"three\" => 3})\n\nThere is an unavoidable overhead compared to natively implemented maps, so\nkeep using regular maps when you do not care about the insertion order.\n\n`Aja.OrdMap`:\n- provides efficient (logarithmic) access: it is not a simple list of tuples\n- implements the `Access` behaviour, `Enum` / `Inspect` / `Collectable` protocols\n- optionally implements the `Jason.Encoder` protocol if `Jason` is installed","ref":"Aja.OrdMap.html"},{"type":"module","title":"Examples - Aja.OrdMap","doc":"`Aja.OrdMap` offers the same API as `Map` :\n\n    iex> ord_map = Aja.OrdMap.new([b: \"Bat\", a: \"Ant\", c: \"Cat\"])\n    ord(%{b: \"Bat\", a: \"Ant\", c: \"Cat\"})\n    iex> Aja.OrdMap.get(ord_map, :c)\n    \"Cat\"\n    iex> Aja.OrdMap.fetch(ord_map, :a)\n    {:ok, \"Ant\"}\n    iex> Aja.OrdMap.put(ord_map, :d, \"Dinosaur\")\n    ord(%{b: \"Bat\", a: \"Ant\", c: \"Cat\", d: \"Dinosaur\"})\n    iex> Aja.OrdMap.put(ord_map, :b, \"Buffalo\")\n    ord(%{b: \"Buffalo\", a: \"Ant\", c: \"Cat\"})\n    iex> Enum.to_list(ord_map)\n    [b: \"Bat\", a: \"Ant\", c: \"Cat\"]\n    iex> [d: \"Dinosaur\", b: \"Buffalo\", e: \"Eel\"] |> Enum.into(ord_map)\n    ord(%{b: \"Buffalo\", a: \"Ant\", c: \"Cat\", d: \"Dinosaur\", e: \"Eel\"})","ref":"Aja.OrdMap.html#module-examples"},{"type":"module","title":"Specific functions - Aja.OrdMap","doc":"Due to its ordered nature, `Aja.OrdMap` also offers some extra methods not present in `Map`, like:\n- `first/1` and `last/1` to efficiently retrieve the first / last key-value pair\n- `foldl/3` and `foldr/3` to efficiently fold (reduce) from left-to-right or right-to-left\n\nExamples:\n\n    iex> ord_map = Aja.OrdMap.new(b: \"Bat\", a: \"Ant\", c: \"Cat\")\n    iex> Aja.OrdMap.first(ord_map)\n    {:b, \"Bat\"}\n    iex> Aja.OrdMap.last(ord_map)\n    {:c, \"Cat\"}\n    iex> Aja.OrdMap.foldr(ord_map, [], fn {_key, value}, acc -> [value <> \"man\" | acc] end)\n    [\"Batman\", \"Antman\", \"Catman\"]","ref":"Aja.OrdMap.html#module-specific-functions"},{"type":"module","title":"Access behaviour - Aja.OrdMap","doc":"`Aja.OrdMap` implements the `Access` behaviour.\n\n    iex> ord_map = Aja.OrdMap.new([a: \"Ant\", b: \"Bat\", c: \"Cat\"])\n    iex> ord_map[:a]\n    \"Ant\"\n    iex> put_in(ord_map[:b], \"Buffalo\")\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> put_in(ord_map[:d], \"Dinosaur\")\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\", d: \"Dinosaur\"})\n    iex> {\"Cat\", updated} = pop_in(ord_map[:c]); updated\n    ord(%{a: \"Ant\", b: \"Bat\"})","ref":"Aja.OrdMap.html#module-access-behaviour"},{"type":"module","title":"Convenience [`ord/1`](`Aja.ord/1`) and [`ord_size/1`](`Aja.ord_size/1`) macros - Aja.OrdMap","doc":"The `Aja.OrdMap` module can be used without any macro.\n\nThe `Aja.ord/1` macro does however provide some syntactic sugar to make\nit more convenient to work with ordered maps, namely:\n- construct new ordered maps without the clutter of a entry list\n- pattern match on key-values like regular maps\n- update some existing keys\n\nExamples:\n\n    iex> import Aja\n    iex> ord_map = ord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\n    ord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\n    iex> ord(%{\"三\" => three, \"一\" => one}) = ord_map\n    iex> {one, three}\n    {1, 3}\n    iex> ord(%{ord_map | \"二\" => \"NI!\"})\n    ord(%{\"一\" => 1, \"二\" => \"NI!\", \"三\" => 3})\n\nNotes:\n- pattern-matching on keys is not affected by insertion order.\n- For expressions with constant keys, `Aja.ord/1` is able to generate the AST at compile time like the `Aja.vec/1` macro.\n\nThe `Aja.ord_size/1` macro can be used in guards:\n\n    iex> import Aja\n    iex> match?(v when ord_size(v) > 2, ord%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\n    true","ref":"Aja.OrdMap.html#module-convenience-ord-1-aja-ord-1-and-ord_size-1-aja-ord_size-1-macros"},{"type":"module","title":"With `Jason` - Aja.OrdMap","doc":"iex> Aja.OrdMap.new([{\"un\", 1}, {\"deux\", 2}, {\"trois\", 3}]) |> Jason.encode!()\n    \"{\\\"un\\\":1,\\\"deux\\\":2,\\\"trois\\\":3}\"\n\nJSON encoding preserves the insertion order. Comparing with a regular map:\n\n    iex> Map.new([{\"un\", 1}, {\"deux\", 2}, {\"trois\", 3}]) |> Jason.encode!()\n    \"{\\\"deux\\\":2,\\\"trois\\\":3,\\\"un\\\":1}\"\n\nThere is no way as of now to decode JSON using `Aja.OrdMap`.","ref":"Aja.OrdMap.html#module-with-jason"},{"type":"module","title":"Key deletion and sparse maps - Aja.OrdMap","doc":"Due to the underlying structures being used, efficient key deletion implies keeping around some\n\"holes\" to avoid rebuilding the whole structure.\n\nSuch an ord map will be called **sparse**, while an ord map that never had a key deleted will be\nreferred as **dense**.\n\nThe implications of sparse structures are multiple:\n- unlike dense structures, they cannot be compared as erlang terms\n  (using either `==/2`, `===/2` or the pin operator `^`)\n- `Aja.OrdMap.equal?/2` can safely compare both sparse and dense structures, but is slower for sparse\n- enumerating sparse structures is less efficient than dense ones\n\nCalling `Aja.OrdMap.new/1` on a sparse ord map will rebuild a new dense one from scratch (which can be expensive).\n\n    iex> dense = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\")\n    ord(%{a: \"Ant\", b: \"Bat\"})\n    iex> sparse = Aja.OrdMap.new(c: \"Cat\", a: \"Ant\", b: \"Bat\") |> Aja.OrdMap.delete(:c)\n    #Aja.OrdMap<%{a: \"Ant\", b: \"Bat\"}, sparse?: true>\n    iex> dense == sparse\n    false\n    iex> match?(^dense, sparse)\n    false\n    iex> Aja.OrdMap.equal?(dense, sparse)  # works with sparse maps, but less efficient\n    true\n    iex> new_dense = Aja.OrdMap.new(sparse)  # rebuild a dense map from a sparse one\n    ord(%{a: \"Ant\", b: \"Bat\"})\n    iex> new_dense === dense\n    true\n\nIn order to avoid having to worry about memory issues when adding and deleting keys successively,\nord maps cannot be more than half sparse, and are periodically rebuilt as dense upon deletion.\n\n    iex> sparse = Aja.OrdMap.new(c: \"Cat\", a: \"Ant\", b: \"Bat\") |> Aja.OrdMap.delete(:c)\n    #Aja.OrdMap<%{a: \"Ant\", b: \"Bat\"}, sparse?: true>\n    iex> Aja.OrdMap.delete(sparse, :a)\n    ord(%{b: \"Bat\"})\n\nNote: Deleting the last key does not make a dense ord map sparse. This is not a bug,\nbut an expected behavior due to how data is stored.\n\n    iex> Aja.OrdMap.new([one: 1, two: 2, three: 3]) |> Aja.OrdMap.delete(:three)\n    ord(%{one: 1, two: 2})\n\nThe `dense?/1` and `sparse?/1` functions can be used to check if a `Aja.OrdMap` is dense or sparse.\n\nWhile this design puts some burden on the developer, the idea behind it is:\n- to keep it as convenient and performant as possible unless deletion is necessary\n- to be transparent about sparse structures and their limitation\n- instead of constantly rebuild new dense structures, let users decide the best timing to do it\n- still work fine with sparse structures, but in a degraded mode\n- protect users about potential memory leaks and performance issues","ref":"Aja.OrdMap.html#module-key-deletion-and-sparse-maps"},{"type":"module","title":"Pattern-matching and opaque type - Aja.OrdMap","doc":"An `Aja.OrdMap` is represented internally using the `%Aja.OrdMap{}` struct. This struct\ncan be used whenever there's a need to pattern match on something being an `Aja.OrdMap`:\n    iex> match?(%Aja.OrdMap{}, Aja.OrdMap.new())\n    true\n\nNote, however, that `Aja.OrdMap` should be considered an opaque type: its struct internal fields\nmust not be accessed directly (even if not enforced by dialyzer because of pattern-matching).\n\nAs discussed in the previous section, [`ord/1`](`Aja.ord/1`) and [`ord_size/1`](`Aja.ord_size/1`) makes it\npossible to pattern match on keys as well as check the type and size.","ref":"Aja.OrdMap.html#module-pattern-matching-and-opaque-type"},{"type":"module","title":"Memory overhead - Aja.OrdMap","doc":"`Aja.OrdMap` takes roughly 2~3x more memory than a regular map depending on the type of data.\n`:erts_debug.size(map)` can be used to confirm the overhead on a concrete use case.","ref":"Aja.OrdMap.html#module-memory-overhead"},{"type":"function","title":"Aja.OrdMap.delete/2","doc":"Deletes the entry in `ord_map` for a specific `key`.\n\nIf the `key` does not exist, returns `ord_map` unchanged.","ref":"Aja.OrdMap.html#delete/2"},{"type":"function","title":"Examples - Aja.OrdMap.delete/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.delete(ord_map, :b)\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> Aja.OrdMap.delete(ord_map, :z)\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#delete/2-examples"},{"type":"function","title":"Aja.OrdMap.dense?/1","doc":"Returns `true` if `ord_map` is dense; otherwise returns `false`.\n\nSee the [section about sparse structures](#module-key-deletion-and-sparse-maps) for more information.","ref":"Aja.OrdMap.html#dense?/1"},{"type":"function","title":"Examples - Aja.OrdMap.dense?/1","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})\n    iex> Aja.OrdMap.dense?(ord_map)\n    true\n    iex> sparse = Aja.OrdMap.delete(ord_map, :b)\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> Aja.OrdMap.dense?(sparse)\n    false","ref":"Aja.OrdMap.html#dense?/1-examples"},{"type":"function","title":"Aja.OrdMap.drop/2","doc":"Drops the given `keys` from `ord_map`.\n\nIf `keys` contains keys that are not in `ord_map`, they're simply ignored.","ref":"Aja.OrdMap.html#drop/2"},{"type":"function","title":"Examples - Aja.OrdMap.drop/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.drop(ord_map, [:b, :d])\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>","ref":"Aja.OrdMap.html#drop/2-examples"},{"type":"function","title":"Aja.OrdMap.equal?/2","doc":"Checks if two ordered maps are equal, meaning they have the same key-value pairs\nin the same order.","ref":"Aja.OrdMap.html#equal?/2"},{"type":"function","title":"Examples - Aja.OrdMap.equal?/2","doc":"iex> Aja.OrdMap.equal?(Aja.OrdMap.new(a: 1, b: 2), Aja.OrdMap.new(a: 1, b: 2))\n    true\n    iex> Aja.OrdMap.equal?(Aja.OrdMap.new(a: 1, b: 2), Aja.OrdMap.new(b: 2, a: 1))\n    false\n    iex> Aja.OrdMap.equal?(Aja.OrdMap.new(a: 1, b: 2), Aja.OrdMap.new(a: 3, b: 2))\n    false","ref":"Aja.OrdMap.html#equal?/2-examples"},{"type":"function","title":"Aja.OrdMap.fetch/2","doc":"Fetches the value for a specific `key` and returns it in a ok-entry.\nIf the key does not exist, returns :error.","ref":"Aja.OrdMap.html#fetch/2"},{"type":"function","title":"Examples - Aja.OrdMap.fetch/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"A\", b: \"B\", c: \"C\")\n    iex> Aja.OrdMap.fetch(ord_map, :c)\n    {:ok, \"C\"}\n    iex> Aja.OrdMap.fetch(ord_map, :z)\n    :error","ref":"Aja.OrdMap.html#fetch/2-examples"},{"type":"function","title":"Aja.OrdMap.fetch!/2","doc":"Fetches the value for a specific `key` in the given `ord_map`,\nerroring out if `ord_map` doesn't contain `key`.\n\nIf `ord_map` doesn't contain `key`, a `KeyError` exception is raised.","ref":"Aja.OrdMap.html#fetch!/2"},{"type":"function","title":"Examples - Aja.OrdMap.fetch!/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"A\", b: \"B\", c: \"C\")\n    iex> Aja.OrdMap.fetch!(ord_map, :c)\n    \"C\"\n    iex> Aja.OrdMap.fetch!(ord_map, :z)\n    ** (KeyError) key :z not found in: ord(%{a: \"A\", b: \"B\", c: \"C\"})","ref":"Aja.OrdMap.html#fetch!/2-examples"},{"type":"function","title":"Aja.OrdMap.filter/2","doc":"Returns a new ordered map containing only those pairs from `ord_map` for which `fun` returns a truthy value.\n\n`fun` receives the key and value of each of the elements in `ord_map` as a key-value pair.\nPreserves the order of `ord_map`.\n\nMirrors `Map.filter/2`.\nSee also `reject/2` which discards all elements where the function returns a truthy value.","ref":"Aja.OrdMap.html#filter/2"},{"type":"function","title":"Examples - Aja.OrdMap.filter/2","doc":"iex> ord_map = Aja.OrdMap.new([three: 3, two: 2, one: 1, zero: 0])\n    iex> Aja.OrdMap.filter(ord_map, fn {_key, val} -> rem(val, 2) == 1 end)\n    ord(%{three: 3, one: 1})","ref":"Aja.OrdMap.html#filter/2-examples"},{"type":"function","title":"Aja.OrdMap.first/2","doc":"Finds the fist `{key, value}` pair in `ord_map`.\n\nReturns a `{key, value}` entry if `ord_map` is non-empty, or `nil` else.","ref":"Aja.OrdMap.html#first/2"},{"type":"function","title":"Examples - Aja.OrdMap.first/2","doc":"iex> Aja.OrdMap.new([b: \"B\", d: \"D\", a: \"A\", c: \"C\"]) |> Aja.OrdMap.first()\n    {:b, \"B\"}\n    iex> Aja.OrdMap.new([]) |> Aja.OrdMap.first()\n    nil\n    iex> Aja.OrdMap.new([]) |> Aja.OrdMap.first(:error)\n    :error","ref":"Aja.OrdMap.html#first/2-examples"},{"type":"function","title":"Aja.OrdMap.foldl/3","doc":"Folds (reduces) the given `ord_map` from the left with the function `fun`.\nRequires an accumulator `acc`.","ref":"Aja.OrdMap.html#foldl/3"},{"type":"function","title":"Examples - Aja.OrdMap.foldl/3","doc":"iex> ord_map = Aja.OrdMap.new([b: \"Bat\", c: \"Cat\", a: \"Ant\"])\n    iex> Aja.OrdMap.foldl(ord_map, \"\", fn {_key, value}, acc -> value <> acc end)\n    \"AntCatBat\"\n    iex> Aja.OrdMap.foldl(ord_map, [], fn {key, value}, acc -> [{key, value <> \"man\"} | acc] end)\n    [a: \"Antman\", c: \"Catman\", b: \"Batman\"]","ref":"Aja.OrdMap.html#foldl/3-examples"},{"type":"function","title":"Aja.OrdMap.foldr/3","doc":"Folds (reduces) the given `ord_map` from the right with the function `fun`.\nRequires an accumulator `acc`.\n\nUnlike linked lists, this is as efficient as `foldl/3`. This can typically save a call\nto `Enum.reverse/1` on the result when building a list.","ref":"Aja.OrdMap.html#foldr/3"},{"type":"function","title":"Examples - Aja.OrdMap.foldr/3","doc":"iex> ord_map = Aja.OrdMap.new([b: \"Bat\", c: \"Cat\", a: \"Ant\"])\n    iex> Aja.OrdMap.foldr(ord_map, \"\", fn {_key, value}, acc -> value <> acc end)\n    \"BatCatAnt\"\n    iex> Aja.OrdMap.foldr(ord_map, [], fn {key, value}, acc -> [{key, value <> \"man\"} | acc] end)\n    [b: \"Batman\", c: \"Catman\", a: \"Antman\"]","ref":"Aja.OrdMap.html#foldr/3-examples"},{"type":"function","title":"Aja.OrdMap.from_keys/2","doc":"Builds an ordered map from the given `keys` list and the fixed `value`.\n\nPreserves the order of `keys`.","ref":"Aja.OrdMap.html#from_keys/2"},{"type":"function","title":"Examples - Aja.OrdMap.from_keys/2","doc":"iex> Aja.OrdMap.from_keys([:c, :a, :d, :b], 0)\n    ord(%{c: 0, a: 0, d: 0, b: 0})","ref":"Aja.OrdMap.html#from_keys/2-examples"},{"type":"function","title":"Aja.OrdMap.from_struct/1","doc":"Converts a `struct` to an ordered map.\n\nIt accepts the struct module or a struct itself and\nsimply removes the `__struct__` field from the given struct\nor from a new struct generated from the given module.\n\nRespects the field order in Elixir >= 1.14.","ref":"Aja.OrdMap.html#from_struct/1"},{"type":"function","title":"Example - Aja.OrdMap.from_struct/1","doc":"defmodule User do\n      defstruct [:name, :age]\n    end\n\n    Aja.OrdMap.from_struct(User)\n    ord(%{name: nil, age: nil})\n\n    Aja.OrdMap.from_struct(%User{name: \"john\", age: 44})\n    ord(%{name: \"john\", age: 44})","ref":"Aja.OrdMap.html#from_struct/1-example"},{"type":"function","title":"Aja.OrdMap.get/3","doc":"Gets the value for a specific `key` in `ord_map`.\n\nIf `key` is present in `ord_map` then its value `value` is\nreturned. Otherwise, `default` is returned.\n\nIf `default` is not provided, `nil` is used.","ref":"Aja.OrdMap.html#get/3"},{"type":"function","title":"Examples - Aja.OrdMap.get/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.get(ord_map, :a)\n    \"Ant\"\n    iex> Aja.OrdMap.get(ord_map, :z)\n    nil\n    iex> Aja.OrdMap.get(ord_map, :z, \"Zebra\")\n    \"Zebra\"","ref":"Aja.OrdMap.html#get/3-examples"},{"type":"function","title":"Aja.OrdMap.get_and_update/3","doc":"Gets the value from `key` and updates it, all in one pass.\n\nMirrors `Map.get_and_update/3`, see its documentation.","ref":"Aja.OrdMap.html#get_and_update/3"},{"type":"function","title":"Examples - Aja.OrdMap.get_and_update/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> {\"bat\", updated} = Aja.OrdMap.get_and_update(ord_map, :b, fn current_value ->\n    ...>   {current_value && String.downcase(current_value), \"Buffalo\"}\n    ...> end)\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> {nil, updated} = Aja.OrdMap.get_and_update(ord_map, :z, fn current_value ->\n    ...>   {current_value && String.downcase(current_value), \"Zebra\"}\n    ...> end)\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\", z: \"Zebra\"})\n    iex> {\"Bat\", updated} = Aja.OrdMap.get_and_update(ord_map, :b, fn _ -> :pop end)\n    iex> updated\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> {nil, updated} = Aja.OrdMap.get_and_update(ord_map, :z, fn _ -> :pop end)\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#get_and_update/3-examples"},{"type":"function","title":"Aja.OrdMap.get_and_update!/3","doc":"Gets the value from `key` and updates it, all in one pass.\n\nMirrors `Map.get_and_update!/3`, see its documentation.","ref":"Aja.OrdMap.html#get_and_update!/3"},{"type":"function","title":"Examples - Aja.OrdMap.get_and_update!/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> {\"bat\", updated} = Aja.OrdMap.get_and_update!(ord_map, :b, fn current_value ->\n    ...>   {current_value && String.downcase(current_value), \"Buffalo\"}\n    ...> end)\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> Aja.OrdMap.get_and_update!(ord_map, :z, fn current_value ->\n    ...>   {current_value && String.downcase(current_value), \"Zebra\"}\n    ...> end)\n    ** (KeyError) key :z not found in: ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#get_and_update!/3-examples"},{"type":"function","title":"Aja.OrdMap.get_lazy/3","doc":"Gets the value for a specific `key` in `ord_map`.\n\nIf `key` is present in `ord_map` then its value `value` is\nreturned. Otherwise, `fun` is evaluated and its result is returned.\n\nThis is useful if the default value is very expensive to calculate or\ngenerally difficult to setup and teardown again.","ref":"Aja.OrdMap.html#get_lazy/3"},{"type":"function","title":"Examples - Aja.OrdMap.get_lazy/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> expensive_fun = fn -> \"Zebra\" end\n    iex> Aja.OrdMap.get_lazy(ord_map, :a, expensive_fun)\n    \"Ant\"\n    iex> Aja.OrdMap.get_lazy(ord_map, :z, expensive_fun)\n    \"Zebra\"","ref":"Aja.OrdMap.html#get_lazy/3-examples"},{"type":"function","title":"Aja.OrdMap.has_key?/2","doc":"Returns whether the given `key` exists in `ord_map`.","ref":"Aja.OrdMap.html#has_key?/2"},{"type":"function","title":"Examples - Aja.OrdMap.has_key?/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.has_key?(ord_map, :a)\n    true\n    iex> Aja.OrdMap.has_key?(ord_map, :d)\n    false","ref":"Aja.OrdMap.html#has_key?/2-examples"},{"type":"function","title":"Aja.OrdMap.keys/1","doc":"Returns all keys from `ord_map`.","ref":"Aja.OrdMap.html#keys/1"},{"type":"function","title":"Examples - Aja.OrdMap.keys/1","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", c: \"Cat\", a: \"Ant\")\n    iex> Aja.OrdMap.keys(ord_map)\n    [:b, :c, :a]","ref":"Aja.OrdMap.html#keys/1-examples"},{"type":"function","title":"Aja.OrdMap.last/2","doc":"Finds the last `{key, value}` pair in `ord_map`.\n\nReturns a `{key, value}` entry if `ord_map` is non-empty, or `nil` else.\nCan be accessed efficiently due to the underlying vector.","ref":"Aja.OrdMap.html#last/2"},{"type":"function","title":"Examples - Aja.OrdMap.last/2","doc":"iex> Aja.OrdMap.new([b: \"B\", d: \"D\", a: \"A\", c: \"C\"]) |> Aja.OrdMap.last()\n    {:c, \"C\"}\n    iex> Aja.OrdMap.new([]) |> Aja.OrdMap.last()\n    nil\n    iex> Aja.OrdMap.new([]) |> Aja.OrdMap.last(:error)\n    :error","ref":"Aja.OrdMap.html#last/2-examples"},{"type":"function","title":"Aja.OrdMap.merge/2","doc":"Merges a map or an ordered map into an `ord_map`.\n\nAll keys in `map_or_ord_map` will be added to `ord_map`, overriding any existing one\n(i.e., the keys in `map_or_ord_map` \"have precedence\" over the ones in `ord_map`).","ref":"Aja.OrdMap.html#merge/2"},{"type":"function","title":"Examples - Aja.OrdMap.merge/2","doc":"iex> Aja.OrdMap.merge(Aja.OrdMap.new(%{a: 1, b: 2}), Aja.OrdMap.new(%{a: 3, d: 4}))\n    ord(%{a: 3, b: 2, d: 4})\n    iex> Aja.OrdMap.merge(Aja.OrdMap.new(%{a: 1, b: 2}), %{a: 3, d: 4})\n    ord(%{a: 3, b: 2, d: 4})","ref":"Aja.OrdMap.html#merge/2-examples"},{"type":"function","title":"Aja.OrdMap.new/0","doc":"Returns a new empty ordered map.","ref":"Aja.OrdMap.html#new/0"},{"type":"function","title":"Examples - Aja.OrdMap.new/0","doc":"iex> Aja.OrdMap.new()\n    ord(%{})","ref":"Aja.OrdMap.html#new/0-examples"},{"type":"function","title":"Aja.OrdMap.new/1","doc":"Creates an ordered map from an `enumerable`.\n\nPreserves the original order of keys.\nDuplicated keys are removed; the latest one prevails.","ref":"Aja.OrdMap.html#new/1"},{"type":"function","title":"Examples - Aja.OrdMap.new/1","doc":"iex> Aja.OrdMap.new(b: \"Bat\", a: \"Ant\", c: \"Cat\")\n    ord(%{b: \"Bat\", a: \"Ant\", c: \"Cat\"})\n    iex> Aja.OrdMap.new(b: \"Bat\", a: \"Ant\", b: \"Buffalo\", a: \"Antelope\")\n    ord(%{b: \"Buffalo\", a: \"Antelope\"})\n\n`new/1` will return dense ord maps untouched, but will rebuild sparse ord maps from scratch.\nThis can be used to build a dense ord map from from a sparse one.\nSee the [section about sparse structures](#module-key-deletion-and-sparse-maps) for more information.\n\n    iex> sparse = Aja.OrdMap.new(c: \"Cat\", a: \"Ant\", b: \"Bat\") |> Aja.OrdMap.delete(:c)\n    #Aja.OrdMap<%{a: \"Ant\", b: \"Bat\"}, sparse?: true>\n    iex> Aja.OrdMap.new(sparse)\n    ord(%{a: \"Ant\", b: \"Bat\"})","ref":"Aja.OrdMap.html#new/1-examples"},{"type":"function","title":"Aja.OrdMap.new/2","doc":"Creates an ordered map from an `enumerable` via the given `transform` function.\n\nPreserves the original order of keys.\nDuplicated keys are removed; the latest one prevails.","ref":"Aja.OrdMap.html#new/2"},{"type":"function","title":"Examples - Aja.OrdMap.new/2","doc":"iex> Aja.OrdMap.new([:a, :b], fn x -> {x, x} end)\n    ord(%{a: :a, b: :b})","ref":"Aja.OrdMap.html#new/2-examples"},{"type":"function","title":"Aja.OrdMap.pop/3","doc":"Returns the value for `key` and the updated ordered map without `key`.\n\nIf `key` is present in the ordered map with a value `value`,\n`{value, new_ord_map}` is returned.\nIf `key` is not present in the ordered map, `{default, ord_map}` is returned.","ref":"Aja.OrdMap.html#pop/3"},{"type":"function","title":"Examples - Aja.OrdMap.pop/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> {\"Bat\", updated} = Aja.OrdMap.pop(ord_map, :b)\n    iex> updated\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> {nil, updated} = Aja.OrdMap.pop(ord_map, :z)\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})\n    iex> {\"Z\", updated} = Aja.OrdMap.pop(ord_map, :z, \"Z\")\n    iex> updated\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#pop/3-examples"},{"type":"function","title":"Aja.OrdMap.pop!/2","doc":"Returns the value for `key` and the updated ordered map without `key`.\n\nBehaves the same as `pop/3` but raises if `key` is not present in `ord_map`.","ref":"Aja.OrdMap.html#pop!/2"},{"type":"function","title":"Examples - Aja.OrdMap.pop!/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> {\"Bat\", updated} = Aja.OrdMap.pop!(ord_map, :b)\n    iex> updated\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> Aja.OrdMap.pop!(ord_map, :z)\n    ** (KeyError) key :z not found in: ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#pop!/2-examples"},{"type":"function","title":"Aja.OrdMap.pop_lazy/3","doc":"Lazily returns and removes the value associated with `key` in `ord_map`.\n\nIf `key` is present in `ord_map`, it returns `{value, new_map}` where `value` is the value of\nthe key and `new_map` is the result of removing `key` from `ord_map`. If `key`\nis not present in `ord_map`, `{fun_result, ord_map}` is returned, where `fun_result`\nis the result of applying `fun`.\n\nThis is useful if the default value is very expensive to calculate or\ngenerally difficult to setup and teardown again.","ref":"Aja.OrdMap.html#pop_lazy/3"},{"type":"function","title":"Examples - Aja.OrdMap.pop_lazy/3","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", a: \"Ant\", c: \"Cat\")\n    iex> expensive_fun = fn -> \"Zebra\" end\n    iex> {\"Ant\", updated} = Aja.OrdMap.pop_lazy(ord_map, :a, expensive_fun)\n    iex> updated\n    #Aja.OrdMap<%{b: \"Bat\", c: \"Cat\"}, sparse?: true>\n    iex> {\"Zebra\", not_updated} = Aja.OrdMap.pop_lazy(ord_map, :z, expensive_fun)\n    iex> not_updated\n    ord(%{b: \"Bat\", a: \"Ant\", c: \"Cat\"})","ref":"Aja.OrdMap.html#pop_lazy/3-examples"},{"type":"function","title":"Aja.OrdMap.put/3","doc":"Puts the given `value` under `key` in `ord_map`.\n\nIf the `key` does exist, it overwrites the existing value without\nchanging its current location.","ref":"Aja.OrdMap.html#put/3"},{"type":"function","title":"Examples - Aja.OrdMap.put/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.put(ord_map, :b, \"Buffalo\")\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> Aja.OrdMap.put(ord_map, :d, \"Dinosaur\")\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\", d: \"Dinosaur\"})","ref":"Aja.OrdMap.html#put/3-examples"},{"type":"function","title":"Aja.OrdMap.put_new/3","doc":"Puts the given `value` under `key` unless the entry `key`\nalready exists in `ord_map`.","ref":"Aja.OrdMap.html#put_new/3"},{"type":"function","title":"Examples - Aja.OrdMap.put_new/3","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.put_new(ord_map, :a, \"Ant\")\n    ord(%{b: \"Bat\", c: \"Cat\", a: \"Ant\"})\n    iex> Aja.OrdMap.put_new(ord_map, :b, \"Buffalo\")\n    ord(%{b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#put_new/3-examples"},{"type":"function","title":"Aja.OrdMap.put_new_lazy/3","doc":"Evaluates `fun` and puts the result under `key`\nin `ord_map` unless `key` is already present.\n\nThis function is useful in case you want to compute the value to put under\n`key` only if `key` is not already present, as for example, when the value is expensive to\ncalculate or generally difficult to setup and teardown again.","ref":"Aja.OrdMap.html#put_new_lazy/3"},{"type":"function","title":"Examples - Aja.OrdMap.put_new_lazy/3","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", c: \"Cat\")\n    iex> expensive_fun = fn -> \"Ant\" end\n    iex> Aja.OrdMap.put_new_lazy(ord_map, :a, expensive_fun)\n    ord(%{b: \"Bat\", c: \"Cat\", a: \"Ant\"})\n    iex> Aja.OrdMap.put_new_lazy(ord_map, :b, expensive_fun)\n    ord(%{b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#put_new_lazy/3-examples"},{"type":"function","title":"Aja.OrdMap.reject/2","doc":"Returns a new ordered map excluding the pairs from `ord_map` for which `fun` returns a truthy value.\nPreserves the order of `ord_map`.\n\nMirrors `Map.reject/2`.\nSee also `filter/2`.","ref":"Aja.OrdMap.html#reject/2"},{"type":"function","title":"Examples - Aja.OrdMap.reject/2","doc":"iex> ord_map = Aja.OrdMap.new([zero: 0, one: 1, two: 2, three: 3])\n    iex> Aja.OrdMap.reject(ord_map, fn {_key, val} -> rem(val, 2) == 1 end)\n    ord(%{zero: 0, two: 2})","ref":"Aja.OrdMap.html#reject/2-examples"},{"type":"function","title":"Aja.OrdMap.replace/3","doc":"Puts a value under `key` only if the `key` already exists in `ord_map`.","ref":"Aja.OrdMap.html#replace/3"},{"type":"function","title":"Examples - Aja.OrdMap.replace/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.replace(ord_map, :b, \"Buffalo\")\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> Aja.OrdMap.replace(ord_map, :d, \"Dinosaur\")\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#replace/3-examples"},{"type":"function","title":"Aja.OrdMap.replace!/3","doc":"Puts a value under `key` only if the `key` already exists in `ord_map`.\n\nIf `key` is not present in `ord_map`, a `KeyError` exception is raised.","ref":"Aja.OrdMap.html#replace!/3"},{"type":"function","title":"Examples - Aja.OrdMap.replace!/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.replace!(ord_map, :b, \"Buffalo\")\n    ord(%{a: \"Ant\", b: \"Buffalo\", c: \"Cat\"})\n    iex> Aja.OrdMap.replace!(ord_map, :d, \"Dinosaur\")\n    ** (KeyError) key :d not found in: ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#replace!/3-examples"},{"type":"function","title":"Aja.OrdMap.size/1","doc":"Returns the number of keys in `ord_map`.","ref":"Aja.OrdMap.html#size/1"},{"type":"function","title":"Examples - Aja.OrdMap.size/1","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.size(ord_map)\n    3\n    iex> Aja.OrdMap.size(Aja.OrdMap.new())\n    0","ref":"Aja.OrdMap.html#size/1-examples"},{"type":"function","title":"Aja.OrdMap.sparse?/1","doc":"Returns `true` if `ord_map` is sparse; otherwise returns `false`.\n\nSee the [section about sparse structures](#module-key-deletion-and-sparse-maps) for more information.","ref":"Aja.OrdMap.html#sparse?/1"},{"type":"function","title":"Examples - Aja.OrdMap.sparse?/1","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})\n    iex> Aja.OrdMap.sparse?(ord_map)\n    false\n    iex> sparse = Aja.OrdMap.delete(ord_map, :b)\n    #Aja.OrdMap<%{a: \"Ant\", c: \"Cat\"}, sparse?: true>\n    iex> Aja.OrdMap.sparse?(sparse)\n    true","ref":"Aja.OrdMap.html#sparse?/1-examples"},{"type":"function","title":"Aja.OrdMap.take/2","doc":"Returns a new ordered map with all the key-value pairs in `ord_map` where the key\nis in `keys`.\n\nIf `keys` contains keys that are not in `ord_map`, they're simply ignored.\nRespects the order of the `keys` list.","ref":"Aja.OrdMap.html#take/2"},{"type":"function","title":"Examples - Aja.OrdMap.take/2","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.take(ord_map, [:c, :e, :a])\n    ord(%{c: \"Cat\", a: \"Ant\"})","ref":"Aja.OrdMap.html#take/2-examples"},{"type":"function","title":"Aja.OrdMap.to_list/1","doc":"Returns all key-values pairs from `ord_map` as a list.","ref":"Aja.OrdMap.html#to_list/1"},{"type":"function","title":"Examples - Aja.OrdMap.to_list/1","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", c: \"Cat\", a: \"Ant\")\n    iex> Aja.OrdMap.to_list(ord_map)\n    [b: \"Bat\", c: \"Cat\", a: \"Ant\"]","ref":"Aja.OrdMap.html#to_list/1-examples"},{"type":"function","title":"Aja.OrdMap.update/4","doc":"Puts a value under `key` only if the `key` already exists in `ord_map`.","ref":"Aja.OrdMap.html#update/4"},{"type":"function","title":"Examples - Aja.OrdMap.update/4","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.update(ord_map, :b, \"N/A\", &String.upcase/1)\n    ord(%{a: \"Ant\", b: \"BAT\", c: \"Cat\"})\n    iex> Aja.OrdMap.update(ord_map, :z, \"N/A\", &String.upcase/1)\n    ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\", z: \"N/A\"})","ref":"Aja.OrdMap.html#update/4-examples"},{"type":"function","title":"Aja.OrdMap.update!/3","doc":"Puts a value under `key` only if the `key` already exists in `ord_map`.\n\nIf `key` is not present in `ord_map`, a `KeyError` exception is raised.","ref":"Aja.OrdMap.html#update!/3"},{"type":"function","title":"Examples - Aja.OrdMap.update!/3","doc":"iex> ord_map = Aja.OrdMap.new(a: \"Ant\", b: \"Bat\", c: \"Cat\")\n    iex> Aja.OrdMap.update!(ord_map, :b,  &String.upcase/1)\n    ord(%{a: \"Ant\", b: \"BAT\", c: \"Cat\"})\n    iex> Aja.OrdMap.update!(ord_map, :d, &String.upcase/1)\n    ** (KeyError) key :d not found in: ord(%{a: \"Ant\", b: \"Bat\", c: \"Cat\"})","ref":"Aja.OrdMap.html#update!/3-examples"},{"type":"function","title":"Aja.OrdMap.values/1","doc":"Returns all values from `ord_map`.","ref":"Aja.OrdMap.html#values/1"},{"type":"function","title":"Examples - Aja.OrdMap.values/1","doc":"iex> ord_map = Aja.OrdMap.new(b: \"Bat\", c: \"Cat\", a: \"Ant\")\n    iex> Aja.OrdMap.values(ord_map)\n    [\"Bat\", \"Cat\", \"Ant\"]","ref":"Aja.OrdMap.html#values/1-examples"},{"type":"type","title":"Aja.OrdMap.key/0","doc":"","ref":"Aja.OrdMap.html#t:key/0"},{"type":"type","title":"Aja.OrdMap.t/0","doc":"","ref":"Aja.OrdMap.html#t:t/0"},{"type":"type","title":"Aja.OrdMap.t/2","doc":"The type of an `Aja.OrdMap` with keys of the type `key` and values of the type `value`.\n\nIt should be considered opaque even though it isn't enforced by dialyzer to enable pattern-matching.","ref":"Aja.OrdMap.html#t:t/2"},{"type":"type","title":"Aja.OrdMap.value/0","doc":"","ref":"Aja.OrdMap.html#t:value/0"},{"type":"module","title":"Aja.Pair","doc":"Convenience helpers for working with `{atom, value}` tuples without breaking the pipe.","ref":"Aja.Pair.html"},{"type":"function","title":"Aja.Pair.unwrap!/2","doc":"Unwraps an idiomatic `{atom, value}` tuple when the atom is what is being expected.\nConvenient for not breaking the pipe.","ref":"Aja.Pair.html#unwrap!/2"},{"type":"function","title":"Examples - Aja.Pair.unwrap!/2","doc":"iex> {:ok, 55} |> Aja.Pair.unwrap!(:ok)\n    55\n    iex> :error |> Aja.Pair.unwrap!(:ok)\n    ** (ArgumentError) unwrap!/2 expected {:ok, _}, got: :error","ref":"Aja.Pair.html#unwrap!/2-examples"},{"type":"function","title":"Aja.Pair.wrap/2","doc":"Wraps the `value` as an idiomatic `{atom, value}` tuple.\nConvenient for not breaking the pipe.","ref":"Aja.Pair.html#wrap/2"},{"type":"function","title":"Examples - Aja.Pair.wrap/2","doc":"iex> 55 |> Aja.Pair.wrap(:ok)\n    {:ok, 55}\n    iex> %{a: 5} |> Map.update!(:a, & &1 + 1) |> Aja.Pair.wrap(:no_reply)\n    {:no_reply, %{a: 6}}","ref":"Aja.Pair.html#wrap/2-examples"},{"type":"module","title":"Aja.String","doc":"Some extra helper functions for working with strings,\nthat are not in the core `String` module.","ref":"Aja.String.html"},{"type":"function","title":"Aja.String.slugify/2","doc":"Transforms the text as a slug. Removes whitespace, special characters and\nconverts the rest to lowercase.\n\nThis is typically useful to generate URLs based on content, e.g. the title of an article.\n\nLike `String.downcase/2`, `slugify/2` also can handle the following modes:\n`:default` (keeps unicode), `:ascii` or `:greek`.","ref":"Aja.String.html#slugify/2"},{"type":"function","title":"Examples - Aja.String.slugify/2","doc":"iex> Aja.String.slugify(\"> \\\"It Was Me, Dio!!!\\\"\\n\")\n    \"it-was-me-dio\"\n    iex> Aja.String.slugify(\"Joseph Joestar a.k.a ジョジョ\")\n    \"joseph-joestar-aka-ジョジョ\"\n    iex> Aja.String.slugify(<<220>>)\n    ** (ArgumentError) Invalid string <<220>>\n\n`:ascii` converts to ascii when possible or strips characters:\n\n    iex> Aja.String.slugify(\"OLÁ!\\n\", :ascii)\n    \"ola\"\n    iex> Aja.String.slugify(\"DIOの世界 -さらば友よ- \", :ascii)\n    \"dio\"\n\n`:greek` handles the context sensitive sigma in Greek:\n\n    iex> Aja.String.slugify(\"\\tΣΣ?\")\n    \"σσ\"\n    iex> Aja.String.slugify(\"\\tΣΣ?\", :greek)\n    \"σς\"","ref":"Aja.String.html#slugify/2-examples"},{"type":"type","title":"Aja.String.mode/0","doc":"","ref":"Aja.String.html#t:mode/0"},{"type":"module","title":"Aja.Vector","doc":"Fast persistent vector with efficient appends and random access.\n\nPersistent vectors have been introduced by Clojure as an efficient alternative to lists.\nMany operations for `Aja.Vector` run in effective constant time (length, random access, appends...),\nunlike linked lists for which most operations run in linear time.\nFunctions that need to go through the whole collection like `map/2` or `foldl/3` are as often fast as\ntheir list equivalents, or sometimes even slightly faster.\n\nVectors also use less memory than lists for \"big\" collections (see the [Memory usage section](#module-memory-usage)).\n\nMake sure to read the [Efficiency guide section](#module-efficiency-guide) to get the best performance\nout of vectors.\n\nErlang's [`:array`](`:array`) module offer similar functionalities.\nHowever `Aja.Vector`:\n- is a better Elixir citizen: pipe-friendliness, `Access` behaviour, `Enum` / `Inspect` / `Collectable` protocols\n- is heavily optimized and should offer higher performance in most use cases, especially \"loops\" like `map/2` / `to_list/1` / `foldl/3`\n- mirrors most of the `Enum` module API (together with `Aja.Enum`) with highly optimized versions for vectors (`Aja.Enum.join/1`, `Aja.Enum.sum/1`, `Aja.Enum.random/1`...)\n- supports negative indexing (e.g. `-1` corresponds to the last element)\n- optionally implements the `Jason.Encoder` protocol if `Jason` is installed\n\nNote: most of the design is inspired by\n[this series of blog posts](https://hypirion.com/musings/understanding-persistent-vector-pt-1) describing\nthe Clojure implementation, but a branching factor of `16 = 2 ^ 4` has been picked instead of `32 = 2 ^ 5`.\nThis choice was made following performance benchmarking that showed better overall performance\nfor this particular implementation.","ref":"Aja.Vector.html"},{"type":"module","title":"Examples - Aja.Vector","doc":"iex> vector = Aja.Vector.new(1..10)\n    vec([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])\n    iex> Aja.Vector.append(vector, :foo)\n    vec([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, :foo])\n    iex> vector[3]\n    4\n    iex> Aja.Vector.replace_at(vector, -1, :bar)\n    vec([1, 2, 3, 4, 5, 6, 7, 8, 9, :bar])\n    iex> 3 in vector\n    true","ref":"Aja.Vector.html#module-examples"},{"type":"module","title":"Access behaviour - Aja.Vector","doc":"`Aja.Vector` implements the `Access` behaviour.\n\n    iex> vector = Aja.Vector.new(1..10)\n    iex> vector[3]\n    4\n    iex> put_in(vector[5], :foo)\n    vec([1, 2, 3, 4, 5, :foo, 7, 8, 9, 10])\n    iex> {9, updated} = pop_in(vector[8]); updated\n    vec([1, 2, 3, 4, 5, 6, 7, 8, 10])","ref":"Aja.Vector.html#module-access-behaviour"},{"type":"module","title":"Convenience [`vec/1`](`Aja.vec/1`) and [`vec_size/1`](`Aja.vec_size/1`) macros - Aja.Vector","doc":"The `Aja.Vector` module can be used without any macro.\n\nThe `Aja.vec/1` macro does however provide some syntactic sugar to make\nit more convenient to work with vectors of known size, namely:\n- pattern match on elements for vectors of known size\n- construct new vectors of known size faster, by generating the AST at compile time\n\nExamples:\n\n    iex> import Aja\n    iex> vec([1, 2, 3])\n    vec([1, 2, 3])\n    iex> vec([1, 2, var, _, _, _]) = Aja.Vector.new(1..6); var\n    3\n    iex> vec(first ||| last) = Aja.Vector.new(0..99_999); {first, last}\n    {0, 99999}\n\nThe `Aja.vec_size/1` macro can be used in guards:\n\n    iex> import Aja\n    iex> match?(v when vec_size(v) > 99, Aja.Vector.new(1..100))\n    true","ref":"Aja.Vector.html#module-convenience-vec-1-aja-vec-1-and-vec_size-1-aja-vec_size-1-macros"},{"type":"module","title":"Convenience [`+++/2`](`Aja.+++/2`) operator - Aja.Vector","doc":"The `Aja.+++/2` operator can make appending to a vector more compact by aliasing `Aja.Vector.concat/2`:\n\n    iex> import Aja\n    iex> vec([1, 2, 3]) +++ vec([4, 5])\n    vec([1, 2, 3, 4, 5])","ref":"Aja.Vector.html#module-convenience-2-aja-2-operator"},{"type":"module","title":"Pattern-matching and opaque type - Aja.Vector","doc":"An `Aja.Vector` is represented internally using the `%Aja.Vector{}` struct. This struct\ncan be used whenever there's a need to pattern match on something being an `Aja.Vector`:\n    iex> match?(%Aja.Vector{}, Aja.Vector.new())\n    true\n\nNote, however, that `Aja.Vector` should be considered an opaque type: its struct internal fields\nmust not be accessed directly (even if not enforced by dialyzer because of pattern-matching).\n\nAs discussed in the previous section, [`vec/1`](`Aja.vec/1`) makes it\npossible to pattern match on size and elements as well as checking the type.","ref":"Aja.Vector.html#module-pattern-matching-and-opaque-type"},{"type":"module","title":"Memory usage - Aja.Vector","doc":"Vectors have a small overhead over lists for smaller collections, but are using\nfar less memory for bigger collections:\n\n    iex> memory_for = fn n -> [Enum.to_list(1..n), Aja.Vector.new(1..n)] |> Enum.map(&:erts_debug.size/1) end\n    iex> memory_for.(1)\n    [2, 32]\n    iex> memory_for.(10)\n    [20, 32]\n    iex> memory_for.(100)\n    [200, 151]\n    iex> memory_for.(10_000)\n    [20000, 11371]\n\nIf you need to work with vectors containing mostly the same value, `Aja.Vector.duplicate/2`\nis highly efficient both in time and memory (logarithmic).\nIt minimizes the number of actual copies and reuses the same nested structures under the hood:\n\n    iex> Aja.Vector.duplicate(0, 10_000) |> :erts_debug.size()\n    117\n    iex> Aja.Vector.duplicate(0, 10_000) |> :erts_debug.flat_size()  # when shared over processes / ETS\n    11371\n\nEven a 1B x 1B matrix of the same element costs virtually nothing!\n\n    big_n = 1_000_000_000\n    0 |> Aja.Vector.duplicate(big_n) |> Aja.Vector.duplicate(big_n) |> :erts_debug.size()\n    539","ref":"Aja.Vector.html#module-memory-usage"},{"type":"module","title":"Efficiency guide - Aja.Vector","doc":"If you are using vectors and not lists, chances are that you care about\nperformance. Here are a couple notes about how to use vectors in an optimal\nway. Most functions from this module are highly efficient, those that are not\nwill indicate it in their documentation.\n\nBut remember the golden rule: **in case of doubt, always benchmark**.\n\n#","ref":"Aja.Vector.html#module-efficiency-guide"},{"type":"module","title":"Avoid prepending - Aja.Vector","doc":"Appending is very efficient, but prepending is highly inefficient since the\nwhole array needs to be reconstructed.\n\n**DON'T**\n\n    Aja.Vector.prepend(vector, :foo)\n\n**DO**\n\n    [:foo | list]  # use lists\n    Aja.Vector.append(vector, :foo)\n\n#","ref":"Aja.Vector.html#module-avoid-prepending"},{"type":"module","title":"Avoid deletions - Aja.Vector","doc":"This implementation of persistent vectors has many advantages, but it does\nnot support efficient deletion, with the exception of the last element that\ncan be popped very efficiently (`Aja.Vector.pop_last/1`, `Aja.Vector.delete_last/1`).\n\nDeleting close to the end of the vector using `Aja.Vector.delete_at/2` or\n`Aja.Vector.pop_at/3` is still fairly fast, but deleting near the beginning needs\nto reconstruct most of the vector.\n\nIf you need to be able to delete arbitrary indexes, chances are you should consider\nan alternative data structure.\nAnother possibility could be to use sparse arrays, defining `nil` as a deleted value\n(but then the indexing and size won't reflect this).\n\n**DON'T**\n\n    Aja.Vector.pop_at(vector, 3)\n    Aja.Vector.delete_at(vector, 3)\n    pop_in(vector[3])\n\n**DO**\n\n    Aja.Vector.pop_last(vector)\n    Aja.Vector.delete_last(vector)\n    Aja.Vector.delete_at(vector, -3)  # close to the end\n    Aja.Vector.replace_at(vector, 3, nil)\n\n#","ref":"Aja.Vector.html#module-avoid-deletions"},{"type":"module","title":"Successive appends - Aja.Vector","doc":"If you just need to append all elements of an enumerable, it is more efficient to use\n`Aja.Vector.concat/2` or its alias `Aja.+++/2` than successive calls to `Aja.Vector.append/2`:\n\n**DON'T**\n\n    Enum.reduce(enumerable, vector, fn val, acc -> Aja.Vector.append(acc, val) end)\n\n**DO**\n\n    Aja.Vector.concat(vector, enumerable)\n    vector +++ enumerable\n\n#","ref":"Aja.Vector.html#module-successive-appends"},{"type":"module","title":"Prefer `Aja.Enum` and `Aja.Vector` to `Enum` for vectors - Aja.Vector","doc":"The `Aja.Enum` module reimplements (nearly) all functions from the `Enum` module to offer\noptimal performance when operating on vectors, and should be used over `Enum` functions whenever possible\n(even if `Aja.Vector` implements the `Enumerable` and `Collectable` protocols for convienience):\n\n**DON'T**\n\n    Enum.sum(vector)\n    Enum.to_list(vector)\n    Enum.reduce(vector, [], fun)\n    Enum.into(enumerable, %Aja.Vector.new())\n    Enum.into(enumerable, vector)\n\n**DO**\n\n    Aja.Enum.sum(vector)\n    Aja.Enum.to_list(vector)  # or Aja.Vector.to_list(vector)\n    Aja.Enum.reduce(vector, [], fun)  # or Aja.Vector.foldl(vector, [], fun)\n    Aja.Vector.new(enumerable)\n    Aja.Enum.into(enumerable, vector)\n    # or Aja.Vector.concat(vector, enumerable)\n    # or vector +++ enumerable\n\nAlthough it depends on the function, you can expect a ~10x speed difference.\n\n`for` comprehensions are actually using `Enumerable` as well, so\nthe same advice holds:\n\n**DON'T**\n\n    for value <- vector do\n      do_stuff()\n    end\n\nIf using it in EEx templates, you might want to cast it to a list:\n\n    for value <- Aja.Vector.to_list(vector) do\n      do_stuff()\n    end\n\n#","ref":"Aja.Vector.html#module-prefer-aja-enum-and-aja-vector-to-enum-for-vectors"},{"type":"module","title":"Exceptions: `Enum` optimized functions - Aja.Vector","doc":"`Enum.member?/2` is implemented in an efficient way, so `in/2` is optimal:\n\n**DO**\n\n    33 in vector\n\n#","ref":"Aja.Vector.html#module-exceptions-enum-optimized-functions"},{"type":"module","title":"Slicing optimization - Aja.Vector","doc":"Slicing any subset on the left on the vector using methods from `Aja.Vector` is\nextremely efficient as the vector internals can be reused:\n\n**DO**\n\n    Aja.Vector.take(vector, 10)  # take a positive amount\n    Aja.Vector.drop(vector, -20)  # drop a negative amount\n    Aja.Vector.slice(vector, 0, 10)  # slicing from 0\n    Aja.Vector.slice(vector, 0..-5)  # slicing from 0\n\n### `Aja.Vector` and `Aja.Enum`\n\n- `Aja.Enum` mirrors `Enum` and should return identical results, therefore many functions would return lists\n- `Aja.Vector` mirrors `Enum` functions that are returning lists, but returns vectors instead\n\n      iex> vector = Aja.Vector.new(1..5)\n      iex> Aja.Enum.reverse(vector)\n      [5, 4, 3, 2, 1]\n      iex> Aja.Vector.reverse(vector)\n      vec([5, 4, 3, 2, 1])\n      iex> Aja.Enum.map(vector, & (&1 * 7))\n      [7, 14, 21, 28, 35]\n      iex> Aja.Vector.map(vector, & (&1 * 7))\n      vec([7, 14, 21, 28, 35])\n\n#","ref":"Aja.Vector.html#module-slicing-optimization"},{"type":"module","title":"Additional notes - Aja.Vector","doc":"* If you need to work with vectors containing mostly the same value,\n  use `Aja.Vector.duplicate/2` (more details in the [Memory usage section](#module-memory-usage)).\n\n* If you work with functions returning vectors of known size, you can use\n  the `Aja.vec/1` macro to defer the generation of the AST for the internal\n  structure to compile time instead of runtime.\n\n      Aja.Vector.new([%{foo: a}, %{foo: b}])  # structure created at runtime\n      vec([%{foo: a}, %{foo: b}])  # structure AST defined at compile time","ref":"Aja.Vector.html#module-additional-notes"},{"type":"function","title":"Aja.Vector.append/2","doc":"Appends a `value` at the end of a `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#append/2"},{"type":"function","title":"Examples - Aja.Vector.append/2","doc":"iex> Aja.Vector.new() |> Aja.Vector.append(:foo)\n    vec([:foo])\n    iex> Aja.Vector.new(1..5) |> Aja.Vector.append(:foo)\n    vec([1, 2, 3, 4, 5, :foo])","ref":"Aja.Vector.html#append/2-examples"},{"type":"function","title":"Aja.Vector.at/2","doc":"Finds the element at the given `index` (zero-based).\n\nReturns `default` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#at/2"},{"type":"function","title":"Examples - Aja.Vector.at/2","doc":"iex> Aja.Vector.new(1..1_000) |> Aja.Vector.at(555)\n    556\n    iex> Aja.Vector.new(1..1_000) |> Aja.Vector.at(1_000)\n    nil","ref":"Aja.Vector.html#at/2-examples"},{"type":"function","title":"Aja.Vector.at/3","doc":"","ref":"Aja.Vector.html#at/3"},{"type":"function","title":"Aja.Vector.at!/2","doc":"Finds the element at the given `index` (zero-based).\n\nRaises an `Aja.Vector.IndexError` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#at!/2"},{"type":"function","title":"Examples - Aja.Vector.at!/2","doc":"iex> Aja.Vector.new(1..1_000) |> Aja.Vector.at!(555)\n    556\n    iex> Aja.Vector.new(1..1_000) |> Aja.Vector.at!(-10)\n    991\n    iex> Aja.Vector.new(1..1_000) |> Aja.Vector.at!(1_000)\n    ** (Aja.Vector.IndexError) out of bound index: 1000 not in -1000..999","ref":"Aja.Vector.html#at!/2-examples"},{"type":"function","title":"Aja.Vector.concat/2","doc":"Appends all values from an `enumerable` at the end of a `vector`.\n\nRuns in effective linear time in respect with the length of `enumerable`,\ndisregarding the size of the `vector`.","ref":"Aja.Vector.html#concat/2"},{"type":"function","title":"Examples - Aja.Vector.concat/2","doc":"iex> Aja.Vector.new(1..5) |> Aja.Vector.concat(10..15)\n    vec([1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15])\n    iex> Aja.Vector.new() |> Aja.Vector.concat(10..15)\n    vec([10, 11, 12, 13, 14, 15])","ref":"Aja.Vector.html#concat/2-examples"},{"type":"function","title":"Aja.Vector.dedup/1","doc":"Returns a copy of the `vector` where all consecutive duplicated elements are collapsed to a single element.\n\nElements are compared using `===/2`.\n\nIf you want to remove all duplicated elements, regardless of order, see `uniq/1`.","ref":"Aja.Vector.html#dedup/1"},{"type":"function","title":"Examples - Aja.Vector.dedup/1","doc":"iex> Aja.Vector.new([1, 2, 3, 3, 2, 1]) |> Aja.Vector.dedup()\n    vec([1, 2, 3, 2, 1])\n    iex> Aja.Vector.new([1, 1, 2, 2.0, :three, :three]) |> Aja.Vector.dedup()\n    vec([1, 2, 2.0, :three])","ref":"Aja.Vector.html#dedup/1-examples"},{"type":"function","title":"Aja.Vector.dedup_by/2","doc":"Returns a copy of the `vector` where all consecutive duplicated elements are collapsed to a single element.\n\nThe function `fun` maps every element to a term which is used to determine if two elements are duplicates.","ref":"Aja.Vector.html#dedup_by/2"},{"type":"function","title":"Examples - Aja.Vector.dedup_by/2","doc":"iex> vector = Aja.Vector.new([{1, :a}, {2, :b}, {2, :c}, {1, :a}])\n    iex> Aja.Vector.dedup_by(vector, fn {x, _} -> x end)\n    vec([{1, :a}, {2, :b}, {1, :a}])\n\n    iex> vector = Aja.Vector.new([5, 1, 2, 3, 2, 1])\n    iex> Aja.Vector.dedup_by(vector, fn x -> x > 2 end)\n    vec([5, 1, 3, 2])","ref":"Aja.Vector.html#dedup_by/2-examples"},{"type":"function","title":"Aja.Vector.delete_at/2","doc":"(Inefficient) Returns a copy of `vector` without the value at the specified `index`.\n\nReturns the `vector` untouched if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in linear time. Its usage is discouraged, see the\n[Efficiency guide](#module-efficiency-guide).","ref":"Aja.Vector.html#delete_at/2"},{"type":"function","title":"Examples - Aja.Vector.delete_at/2","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> Aja.Vector.delete_at(vector, 4)\n    vec([1, 2, 3, 4, 6, 7, 8])\n    iex> Aja.Vector.delete_at(vector, -9)\n    vec([1, 2, 3, 4, 5, 6, 7, 8])","ref":"Aja.Vector.html#delete_at/2-examples"},{"type":"function","title":"Aja.Vector.delete_at!/2","doc":"(Inefficient) Returns a copy of `vector` without the value at the specified `index`.\n\nRaises an `Aja.Vector.IndexError` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in linear time. Its usage is discouraged, see the\n[Efficiency guide](#module-efficiency-guide).","ref":"Aja.Vector.html#delete_at!/2"},{"type":"function","title":"Examples - Aja.Vector.delete_at!/2","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> Aja.Vector.delete_at!(vector, 4)\n    vec([1, 2, 3, 4, 6, 7, 8])\n    iex> Aja.Vector.delete_at!(vector, -9)\n    ** (Aja.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"Aja.Vector.html#delete_at!/2-examples"},{"type":"function","title":"Aja.Vector.delete_last/1","doc":"Removes the last value from the `vector` and returns the updated vector.\n\nLeaves the `vector` untouched if empty.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#delete_last/1"},{"type":"function","title":"Examples - Aja.Vector.delete_last/1","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> Aja.Vector.delete_last(vector)\n    vec([1, 2, 3, 4, 5, 6, 7])\n    iex> Aja.Vector.delete_last(Aja.Vector.new())\n    vec([])","ref":"Aja.Vector.html#delete_last/1-examples"},{"type":"function","title":"Aja.Vector.delete_last!/1","doc":"Removes the last value from the `vector` and returns the updated vector.\n\nRaises an `Aja.Vector.EmptyError` if empty.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#delete_last!/1"},{"type":"function","title":"Examples - Aja.Vector.delete_last!/1","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> Aja.Vector.delete_last!(vector)\n    vec([1, 2, 3, 4, 5, 6, 7])\n    iex> Aja.Vector.delete_last!(Aja.Vector.new())\n    ** (Aja.Vector.EmptyError) empty vector error","ref":"Aja.Vector.html#delete_last!/1-examples"},{"type":"function","title":"Aja.Vector.drop/2","doc":"Drops the amount of elements from the `vector`.\n\nIf a negative `amount` is given, the amount of last values will be dropped.\n\nTime complexity is:\n- linear when `amount` is positive, as the vector needs to be reconstructed.\n- effective constant time when `amount` is negative, as the vector structure can be shared","ref":"Aja.Vector.html#drop/2"},{"type":"function","title":"Examples - Aja.Vector.drop/2","doc":"iex> Aja.Vector.new(0..15) |> Aja.Vector.drop(10)\n    vec([10, 11, 12, 13, 14, 15])\n    iex> Aja.Vector.new(0..5) |> Aja.Vector.drop(0)\n    vec([0, 1, 2, 3, 4, 5])\n    iex> Aja.Vector.new(0..10) |> Aja.Vector.drop(-5)\n    vec([0, 1, 2, 3, 4, 5])","ref":"Aja.Vector.html#drop/2-examples"},{"type":"function","title":"Aja.Vector.drop_while/2","doc":"Drops elements at the beginning of the `vector` while `fun` returns a truthy value.\n\nRuns in linear time.","ref":"Aja.Vector.html#drop_while/2"},{"type":"function","title":"Examples - Aja.Vector.drop_while/2","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.drop_while(fn x -> x < 7 end)\n    vec([7, 8, 9, 10])\n    iex> Aja.Vector.new([1, true, %{}, nil, \"abc\"]) |> Aja.Vector.drop_while(fn x -> x end)\n    vec([nil, \"abc\"])","ref":"Aja.Vector.html#drop_while/2-examples"},{"type":"function","title":"Aja.Vector.duplicate/2","doc":"Duplicates the given element `n` times in a vector.\n\n`n` is an integer greater than or equal to `0`.\nIf `n` is `0`, an empty list is returned.\n\nRuns in logarithmic time regarding `n`. It is very fast and memory efficient\n(see [Memory usage](#module-memory-usage)).","ref":"Aja.Vector.html#duplicate/2"},{"type":"function","title":"Examples - Aja.Vector.duplicate/2","doc":"iex> Aja.Vector.duplicate(nil, 10)\n    vec([nil, nil, nil, nil, nil, nil, nil, nil, nil, nil])\n    iex> Aja.Vector.duplicate(:foo, 0)\n    vec([])","ref":"Aja.Vector.html#duplicate/2-examples"},{"type":"function","title":"Aja.Vector.fetch/2","doc":"Finds the element at the given `index` (zero-based), and returns it in a ok-entry.\nIf the `index` does not exist, returns `:error`.\n\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#fetch/2"},{"type":"function","title":"Examples - Aja.Vector.fetch/2","doc":"iex> Aja.Vector.new(1..1_000) |> Aja.Vector.fetch(555)\n    {:ok, 556}\n    iex> Aja.Vector.new(1..1_000) |> Aja.Vector.fetch(1_000)\n    :error\n    iex> Aja.Vector.new(1..1_000) |> Aja.Vector.fetch(-1)\n    {:ok, 1000}","ref":"Aja.Vector.html#fetch/2-examples"},{"type":"function","title":"Aja.Vector.fetch!/2","doc":"","ref":"Aja.Vector.html#fetch!/2"},{"type":"function","title":"Aja.Vector.filter/2","doc":"Filters the `vector`, i.e. return a new vector containing only elements\nfor which `fun` returns a truthy (neither `false` nor `nil`) value.\n\nRuns in linear time.","ref":"Aja.Vector.html#filter/2"},{"type":"function","title":"Examples - Aja.Vector.filter/2","doc":"iex> vector = Aja.Vector.new(1..100)\n    iex> Aja.Vector.filter(vector, fn i -> rem(i, 13) == 0 end)\n    vec([13, 26, 39, 52, 65, 78, 91])","ref":"Aja.Vector.html#filter/2-examples"},{"type":"function","title":"Aja.Vector.first/2","doc":"Returns the first element in the `vector` or `default` if `vector` is empty.\n\nRuns in actual constant time.","ref":"Aja.Vector.html#first/2"},{"type":"function","title":"Examples - Aja.Vector.first/2","doc":"iex> Aja.Vector.new(1..10_000) |> Aja.Vector.first()\n    1\n    iex> Aja.Vector.new() |> Aja.Vector.first()\n    nil","ref":"Aja.Vector.html#first/2-examples"},{"type":"function","title":"Aja.Vector.flat_map/2","doc":"Maps the given `fun` over `vector` and flattens the result.\n\nThis function returns a new vector built by concatenating the results\nof invoking `fun` on each element of `vector` together.\n\nRuns in linear time.","ref":"Aja.Vector.html#flat_map/2"},{"type":"function","title":"Examples - Aja.Vector.flat_map/2","doc":"iex> Aja.Vector.new(0..4) |> Aja.Vector.flat_map(fn i -> List.duplicate(i, i) end)\n    vec([1, 2, 2, 3, 3, 3, 4, 4, 4, 4])","ref":"Aja.Vector.html#flat_map/2-examples"},{"type":"function","title":"Aja.Vector.foldl/3","doc":"Folds (reduces) the given `vector` from the left with the function `fun`.\nRequires an accumulator `acc`.\n\nRuns in linear time.","ref":"Aja.Vector.html#foldl/3"},{"type":"function","title":"Examples - Aja.Vector.foldl/3","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.foldl(0, &+/2)\n    55\n    iex> Aja.Vector.new(1..10) |> Aja.Vector.foldl([], & [&1 | &2])\n    [10, 9, 8, 7, 6, 5, 4, 3, 2, 1]","ref":"Aja.Vector.html#foldl/3-examples"},{"type":"function","title":"Aja.Vector.foldr/3","doc":"Folds (reduces) the given `vector` from the right with the function `fun`.\nRequires an accumulator `acc`.\n\nUnlike linked lists, this is as efficient as `foldl/3`. This can typically save a call\nto `Enum.reverse/1` on the result when building a list.\n\nRuns in linear time.","ref":"Aja.Vector.html#foldr/3"},{"type":"function","title":"Examples - Aja.Vector.foldr/3","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.foldr(0, &+/2)\n    55\n    iex> Aja.Vector.new(1..10) |> Aja.Vector.foldr([], & [&1 | &2])\n    [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]","ref":"Aja.Vector.html#foldr/3-examples"},{"type":"function","title":"Aja.Vector.get_and_update/3","doc":"Gets the value from key and updates it, all in one pass.\n\nSee `Access.get_and_update/3` for more details.","ref":"Aja.Vector.html#get_and_update/3"},{"type":"function","title":"Examples - Aja.Vector.get_and_update/3","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> {6, updated} = Aja.Vector.get_and_update(vector, 5, fn current_value ->\n    ...>   {current_value, current_value && current_value * 100}\n    ...> end); updated\n    vec([1, 2, 3, 4, 5, 600, 7, 8])\n    iex> {nil, updated} = Aja.Vector.get_and_update(vector, 8, fn current_value ->\n    ...>   {current_value, current_value && current_value * 100}\n    ...> end); updated\n    vec([1, 2, 3, 4, 5, 6, 7, 8])\n    iex> {4, updated} = Aja.Vector.get_and_update(vector, 3, fn _ -> :pop end); updated\n    vec([1, 2, 3, 5, 6, 7, 8])\n    iex> {nil, updated} = Aja.Vector.get_and_update(vector, 8, fn _ -> :pop end); updated\n    vec([1, 2, 3, 4, 5, 6, 7, 8])","ref":"Aja.Vector.html#get_and_update/3-examples"},{"type":"function","title":"Aja.Vector.intersperse/2","doc":"Intersperses `separator` between each element of the `vector`.\n\nRuns in linear time.","ref":"Aja.Vector.html#intersperse/2"},{"type":"function","title":"Examples - Aja.Vector.intersperse/2","doc":"iex> Aja.Vector.new(1..6) |> Aja.Vector.intersperse(nil)\n    vec([1, nil, 2, nil, 3, nil, 4, nil, 5, nil, 6])","ref":"Aja.Vector.html#intersperse/2-examples"},{"type":"function","title":"Aja.Vector.last/2","doc":"Returns the last element in the `vector` or `default` if `vector` is empty.\n\nRuns in constant time (actual, not effective).","ref":"Aja.Vector.html#last/2"},{"type":"function","title":"Examples - Aja.Vector.last/2","doc":"iex> Aja.Vector.new(1..10_000) |> Aja.Vector.last()\n    10_000\n    iex> Aja.Vector.new() |> Aja.Vector.last()\n    nil","ref":"Aja.Vector.html#last/2-examples"},{"type":"function","title":"Aja.Vector.map/2","doc":"Returns a new vector where each element is the result of invoking `fun`\non each corresponding element of `vector`.\n\nRuns in linear time.","ref":"Aja.Vector.html#map/2"},{"type":"function","title":"Examples - Aja.Vector.map/2","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.map(&(&1 * &1))\n    vec([1, 4, 9, 16, 25, 36, 49, 64, 81, 100])","ref":"Aja.Vector.html#map/2-examples"},{"type":"function","title":"Aja.Vector.map_intersperse/3","doc":"Maps and intersperses the `vector` in one pass.\n\nRuns in linear time.","ref":"Aja.Vector.html#map_intersperse/3"},{"type":"function","title":"Examples - Aja.Vector.map_intersperse/3","doc":"iex> Aja.Vector.new(1..6) |> Aja.Vector.map_intersperse(nil, &(&1 * 10))\n    vec([10, nil, 20, nil, 30, nil, 40, nil, 50, nil, 60])","ref":"Aja.Vector.html#map_intersperse/3-examples"},{"type":"function","title":"Aja.Vector.map_reduce/3","doc":"Invokes the given `fun` to each element in the `vector` to reduce\nit to a single element, while keeping an accumulator.\n\nReturns a tuple where the first element is the mapped vector and\nthe second one is the final accumulator.\n\nThe function, `fun`, receives two arguments: the first one is the\nelement, and the second one is the accumulator. `fun` must return\na tuple with two elements in the form of `{result, accumulator}`.","ref":"Aja.Vector.html#map_reduce/3"},{"type":"function","title":"Examples - Aja.Vector.map_reduce/3","doc":"iex> vector = Aja.Vector.new([1, 2, 3])\n    iex> {new_vec, 6} = Aja.Vector.map_reduce(vector, 0, fn x, acc -> {x * 2, x + acc} end)\n    iex> new_vec\n    vec([2, 4, 6])\n\nFor example, if `with_index/2` was not implemented, you could implement it as follows:\n\n    iex> vector = Aja.Vector.new([1, 2, 3])\n    iex> Aja.Vector.map_reduce(vector, 0, fn x, i -> {{x, i}, i + 1} end) |> elem(0)\n    vec([{1, 0}, {2, 1}, {3, 2}])","ref":"Aja.Vector.html#map_reduce/3-examples"},{"type":"function","title":"Aja.Vector.new/0","doc":"Returns a new empty vector.","ref":"Aja.Vector.html#new/0"},{"type":"function","title":"Examples - Aja.Vector.new/0","doc":"iex> Aja.Vector.new()\n    vec([])","ref":"Aja.Vector.html#new/0-examples"},{"type":"function","title":"Aja.Vector.new/1","doc":"Creates a vector from an `enumerable`.\n\nRuns in linear time.","ref":"Aja.Vector.html#new/1"},{"type":"function","title":"Examples - Aja.Vector.new/1","doc":"iex> Aja.Vector.new(10..25)\n    vec([10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25])","ref":"Aja.Vector.html#new/1-examples"},{"type":"function","title":"Aja.Vector.new/2","doc":"Creates a vector from an `enumerable` via the given `transform` function.","ref":"Aja.Vector.html#new/2"},{"type":"function","title":"Examples - Aja.Vector.new/2","doc":"iex> Aja.Vector.new(1..10, &(&1 * &1))\n    vec([1, 4, 9, 16, 25, 36, 49, 64, 81, 100])","ref":"Aja.Vector.html#new/2-examples"},{"type":"function","title":"Aja.Vector.pop_at/3","doc":"(Inefficient) Returns and removes the value at the specified `index` in the `vector`.\n\nReturns the `vector` untouched if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in linear time. Its usage is discouraged, see the\n[Efficiency guide](#module-efficiency-guide).","ref":"Aja.Vector.html#pop_at/3"},{"type":"function","title":"Examples - Aja.Vector.pop_at/3","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> {5, updated} = Aja.Vector.pop_at(vector, 4); updated\n    vec([1, 2, 3, 4, 6, 7, 8])\n    iex> {nil, updated} = Aja.Vector.pop_at(vector, -9); updated\n    vec([1, 2, 3, 4, 5, 6, 7, 8])","ref":"Aja.Vector.html#pop_at/3-examples"},{"type":"function","title":"Aja.Vector.pop_at!/2","doc":"(Inefficient) Returns and removes the value at the specified `index` in the `vector`.\n\nRaises an `Aja.Vector.IndexError` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in linear time. Its usage is discouraged, see the\n[Efficiency guide](#module-efficiency-guide).","ref":"Aja.Vector.html#pop_at!/2"},{"type":"function","title":"Examples - Aja.Vector.pop_at!/2","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> {5, updated} = Aja.Vector.pop_at!(vector, 4); updated\n    vec([1, 2, 3, 4, 6, 7, 8])\n    iex> Aja.Vector.pop_at!(vector, -9)\n    ** (Aja.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"Aja.Vector.html#pop_at!/2-examples"},{"type":"function","title":"Aja.Vector.pop_last/2","doc":"Removes the last value from the `vector` and returns both the value and the updated vector.\n\nLeaves the `vector` untouched if empty.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#pop_last/2"},{"type":"function","title":"Examples - Aja.Vector.pop_last/2","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> {8, updated} = Aja.Vector.pop_last(vector); updated\n    vec([1, 2, 3, 4, 5, 6, 7])\n    iex> {nil, updated} = Aja.Vector.pop_last(Aja.Vector.new()); updated\n    vec([])","ref":"Aja.Vector.html#pop_last/2-examples"},{"type":"function","title":"Aja.Vector.pop_last!/1","doc":"Removes the last value from the `vector` and returns both the value and the updated vector.\n\nRaises an `Aja.Vector.EmptyError` if empty.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#pop_last!/1"},{"type":"function","title":"Examples - Aja.Vector.pop_last!/1","doc":"iex> vector = Aja.Vector.new(1..8)\n    iex> {8, updated} = Aja.Vector.pop_last!(vector); updated\n    vec([1, 2, 3, 4, 5, 6, 7])\n    iex> {nil, updated} = Aja.Vector.pop_last!(Aja.Vector.new()); updated\n    ** (Aja.Vector.EmptyError) empty vector error","ref":"Aja.Vector.html#pop_last!/1-examples"},{"type":"function","title":"Aja.Vector.prepend/2","doc":"(Inefficient) Prepends `value` at the beginning of the `vector`.\n\nRuns in linear time because the whole vector needs to be reconstructuded,\nand should be avoided.","ref":"Aja.Vector.html#prepend/2"},{"type":"function","title":"Examples - Aja.Vector.prepend/2","doc":"iex> Aja.Vector.new() |> Aja.Vector.prepend(:foo)\n    vec([:foo])\n    iex> Aja.Vector.new(1..5) |> Aja.Vector.prepend(:foo)\n    vec([:foo, 1, 2, 3, 4, 5])","ref":"Aja.Vector.html#prepend/2-examples"},{"type":"function","title":"Aja.Vector.reject/2","doc":"Filters the `vector`, i.e. return a new vector containing only elements\nfor which `fun` returns a falsy (either `false` or `nil`) value.\n\nRuns in linear time.","ref":"Aja.Vector.html#reject/2"},{"type":"function","title":"Examples - Aja.Vector.reject/2","doc":"iex> vector = Aja.Vector.new(1..12)\n    iex> Aja.Vector.reject(vector, fn i -> rem(i, 3) == 0 end)\n    vec([1, 2, 4, 5, 7, 8, 10, 11])","ref":"Aja.Vector.html#reject/2-examples"},{"type":"function","title":"Aja.Vector.repeat/2","doc":"Populates a vector of size `n` by calling `generator_fun` repeatedly.","ref":"Aja.Vector.html#repeat/2"},{"type":"function","title":"Examples - Aja.Vector.repeat/2","doc":"# Although not necessary, let's seed the random algorithm\n    iex> :rand.seed(:exrop, {1, 2, 3})\n    iex> Aja.Vector.repeat(&:rand.uniform/0, 3)\n    vec([0.7498295129076106, 0.06161655489244533, 0.7924073127680873])","ref":"Aja.Vector.html#repeat/2-examples"},{"type":"function","title":"Aja.Vector.replace_at/3","doc":"Returns a copy of `vector` with a replaced `value` at the specified `index`.\n\nReturns the `vector` untouched if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#replace_at/3"},{"type":"function","title":"Examples - Aja.Vector.replace_at/3","doc":"iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at(5, :foo)\n    vec([1, 2, 3, 4, 5, :foo, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at(8, :foo)\n    vec([1, 2, 3, 4, 5, 6, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at(-2, :foo)\n    vec([1, 2, 3, 4, 5, 6, :foo, 8])","ref":"Aja.Vector.html#replace_at/3-examples"},{"type":"function","title":"Aja.Vector.replace_at!/3","doc":"Returns a copy of `vector` with a replaced `value` at the specified `index`.\n\nRaises an `Aja.Vector.IndexError` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#replace_at!/3"},{"type":"function","title":"Examples - Aja.Vector.replace_at!/3","doc":"iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at!(5, :foo)\n    vec([1, 2, 3, 4, 5, :foo, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at!(-2, :foo)\n    vec([1, 2, 3, 4, 5, 6, :foo, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.replace_at!(8, :foo)\n    ** (Aja.Vector.IndexError) out of bound index: 8 not in -8..7","ref":"Aja.Vector.html#replace_at!/3-examples"},{"type":"function","title":"Aja.Vector.reverse/1","doc":"Returns the `vector` in reverse order.\n\nRuns in linear time.","ref":"Aja.Vector.html#reverse/1"},{"type":"function","title":"Examples - Aja.Vector.reverse/1","doc":"iex> Aja.Vector.new(1..12) |> Aja.Vector.reverse()\n    vec([12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1])","ref":"Aja.Vector.html#reverse/1-examples"},{"type":"function","title":"Aja.Vector.reverse/2","doc":"Returns the `vector` in reverse order, and concatenates the `tail` (enumerable).\n\nRuns in linear time.","ref":"Aja.Vector.html#reverse/2"},{"type":"function","title":"Examples - Aja.Vector.reverse/2","doc":"iex> Aja.Vector.new(1..5) |> Aja.Vector.reverse(100..105)\n    vec([5, 4, 3, 2, 1, 100, 101, 102, 103, 104, 105])","ref":"Aja.Vector.html#reverse/2-examples"},{"type":"function","title":"Aja.Vector.scan/2","doc":"Applies the given function to each element in the `vector`, storing the result\nin a vector and passing it as the accumulator for the next computation.\n\nUses the first element in the `vector` as the starting value.\n\nRuns in linear time.","ref":"Aja.Vector.html#scan/2"},{"type":"function","title":"Examples - Aja.Vector.scan/2","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.scan(&+/2)\n    vec([1, 3, 6, 10, 15, 21, 28, 36, 45, 55])","ref":"Aja.Vector.html#scan/2-examples"},{"type":"function","title":"Aja.Vector.scan/3","doc":"Applies the given function to each element in the `vector`, storing the result\nin a vector and passing it as the accumulator for the next computation.\n\nUses the given `acc` as the starting value.\n\nRuns in linear time.","ref":"Aja.Vector.html#scan/3"},{"type":"function","title":"Examples - Aja.Vector.scan/3","doc":"iex> Aja.Vector.new(1..10) |> Aja.Vector.scan(100, &+/2)\n    vec([101, 103, 106, 110, 115, 121, 128, 136, 145, 155])","ref":"Aja.Vector.html#scan/3-examples"},{"type":"function","title":"Aja.Vector.shuffle/1","doc":"Returns a new vector with the elements of `vector` shuffled.\n\nSee `Enum.shuffle/1` for notes on implementation and random seed.","ref":"Aja.Vector.html#shuffle/1"},{"type":"function","title":"Examples - Aja.Vector.shuffle/1","doc":"# Although not necessary, let's seed the random algorithm\n    iex> :rand.seed(:exrop, {1, 2, 3})\n    iex> Aja.Vector.new([1, 2, 3]) |> Aja.Vector.shuffle()\n    vec([3, 1, 2])\n    iex> Aja.Vector.new([1, 2, 3]) |> Aja.Vector.shuffle()\n    vec([1, 3, 2])","ref":"Aja.Vector.html#shuffle/1-examples"},{"type":"function","title":"Aja.Vector.size/1","doc":"Returns the number of elements in `vector`.\n\nRuns in constant time.","ref":"Aja.Vector.html#size/1"},{"type":"function","title":"Examples - Aja.Vector.size/1","doc":"iex> Aja.Vector.new(10_000..20_000) |> Aja.Vector.size()\n    10001\n    iex> Aja.Vector.new() |> Aja.Vector.size()\n    0","ref":"Aja.Vector.html#size/1-examples"},{"type":"function","title":"Aja.Vector.slice/2","doc":"Returns a subset of the given `vector` by `index_range`.\n\nWorks the same as `Enum.slice/2`, see its documentation for more details.\n\nRuns in linear time regarding the size of the returned subset.","ref":"Aja.Vector.html#slice/2"},{"type":"function","title":"Examples - Aja.Vector.slice/2","doc":"iex> Aja.Vector.new(0..100) |> Aja.Vector.slice(80..90)\n    vec([80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90])\n    iex> Aja.Vector.new(0..100) |> Aja.Vector.slice(-40..-30//1)\n    vec([61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71])\n    iex> Aja.Vector.new([:only_one]) |> Aja.Vector.slice(0..1000)\n    vec([:only_one])","ref":"Aja.Vector.html#slice/2-examples"},{"type":"function","title":"Aja.Vector.slice/3","doc":"Returns a subset of the given `vector`, from `start_index` (zero-based)\nwith `amount number` of elements if available.\n\nWorks the same as `Enum.slice/3`, see its documentation for more details.\n\nRuns in linear time regarding the size of the returned subset.","ref":"Aja.Vector.html#slice/3"},{"type":"function","title":"Examples - Aja.Vector.slice/3","doc":"iex> Aja.Vector.new(0..100) |> Aja.Vector.slice(80, 10)\n    vec([80, 81, 82, 83, 84, 85, 86, 87, 88, 89])\n    iex> Aja.Vector.new(0..100) |> Aja.Vector.slice(-40, 10)\n    vec([61, 62, 63, 64, 65, 66, 67, 68, 69, 70])\n    iex> Aja.Vector.new([:only_one]) |> Aja.Vector.slice(0, 1000)\n    vec([:only_one])","ref":"Aja.Vector.html#slice/3-examples"},{"type":"function","title":"Aja.Vector.sort/1","doc":"Sorts the `vector` in the same way as `Enum.sort/1`.","ref":"Aja.Vector.html#sort/1"},{"type":"function","title":"Examples - Aja.Vector.sort/1","doc":"iex> Aja.Vector.new(9..1) |> Aja.Vector.sort()\n    vec([1, 2, 3, 4, 5, 6, 7, 8, 9])","ref":"Aja.Vector.html#sort/1-examples"},{"type":"function","title":"Aja.Vector.sort/2","doc":"Sorts the `vector` in the same way as `Enum.sort/2`.\n\nSee `Enum.sort/2` documentation for detailled usage.","ref":"Aja.Vector.html#sort/2"},{"type":"function","title":"Examples - Aja.Vector.sort/2","doc":"iex> Aja.Vector.new(1..9) |> Aja.Vector.sort(:desc)\n    vec([9, 8, 7, 6, 5, 4, 3, 2, 1])","ref":"Aja.Vector.html#sort/2-examples"},{"type":"function","title":"Aja.Vector.sort_by/3","doc":"Sorts the `vector` in the same way as `Enum.sort_by/3`.\n\nSee `Enum.sort_by/3` documentation for detailled usage.","ref":"Aja.Vector.html#sort_by/3"},{"type":"function","title":"Examples - Aja.Vector.sort_by/3","doc":"iex> vector = Aja.Vector.new([\"some\", \"kind\", \"of\", \"monster\"])\n    iex> Aja.Vector.sort_by(vector, &byte_size/1)\n    vec([\"of\", \"some\", \"kind\", \"monster\"])\n    iex> Aja.Vector.sort_by(vector, &{byte_size(&1), String.first(&1)})\n    vec([\"of\", \"kind\", \"some\", \"monster\"])","ref":"Aja.Vector.html#sort_by/3-examples"},{"type":"function","title":"Aja.Vector.split/2","doc":"Splits the `vector` into two vectors, leaving `amount` elements in the first one.\n\nIf `amount` is a negative number, it starts counting from the back to the beginning of the `vector`.\n\nRuns in linear time.","ref":"Aja.Vector.html#split/2"},{"type":"function","title":"Examples - Aja.Vector.split/2","doc":"iex> vector = Aja.Vector.new([1, 2, 3])\n    iex> Aja.Vector.split(vector, 2) |> inspect()\n    \"{vec([1, 2]), vec([3])}\"\n    iex> Aja.Vector.split(vector, 10) |> inspect()\n    \"{vec([1, 2, 3]), vec([])}\"\n    iex> Aja.Vector.split(vector, 0) |> inspect()\n    \"{vec([]), vec([1, 2, 3])}\"\n    iex> Aja.Vector.split(vector, -1) |> inspect()\n    \"{vec([1, 2]), vec([3])}\"\n    iex> Aja.Vector.split(vector, -5) |> inspect()\n    \"{vec([]), vec([1, 2, 3])}\"","ref":"Aja.Vector.html#split/2-examples"},{"type":"function","title":"Aja.Vector.split_while/2","doc":"Splits `vector` in two at the position of the element for which `fun` returns a falsy value\n(`false` or `nil`) for the first time.\n\nIt returns a two-element tuple with two vectors of elements.\nThe element that triggered the split is part of the second vector.\n\nIs basically performing `take_while/2` and `drop_while/2` at once.\n\nRuns in linear time.","ref":"Aja.Vector.html#split_while/2"},{"type":"function","title":"Examples - Aja.Vector.split_while/2","doc":"iex> {taken, dropped} = Aja.Vector.new(1..10) |> Aja.Vector.split_while(fn x -> x < 7 end)\n    iex> taken\n    vec([1, 2, 3, 4, 5, 6])\n    iex> dropped\n    vec([7, 8, 9, 10])","ref":"Aja.Vector.html#split_while/2-examples"},{"type":"function","title":"Aja.Vector.split_with/2","doc":"Splits the `vector` in two vectors according to the given function `fun`.\n\nReturns a tuple with the first vector containing all the elements in `vector`\nfor which applying `fun` returned a truthy value, and a second vector with all\nthe elements for which applying `fun` returned a falsy value (`false` or `nil`).\n\nReturns the same result as `filter/2` and `reject/2` at once, but only walks the\n`vector` once and calls `fun` exactly once per element.\n\nRuns in linear time.","ref":"Aja.Vector.html#split_with/2"},{"type":"function","title":"Examples - Aja.Vector.split_with/2","doc":"iex> vector = Aja.Vector.new(1..12)\n    iex> {filtered, rejected} = Aja.Vector.split_with(vector, fn i -> rem(i, 3) == 0 end)\n    iex> filtered\n    vec([3, 6, 9, 12])\n    iex> rejected\n    vec([1, 2, 4, 5, 7, 8, 10, 11])","ref":"Aja.Vector.html#split_with/2-examples"},{"type":"function","title":"Aja.Vector.take/2","doc":"Takes an `amount` of elements from the beginning or the end of the `vector`.\n\nIf a positive `amount` is given, it takes the amount elements from the beginning of the `vector`.\n\nIf a negative `amount` is given, the amount of elements will be taken from the end.\n\nIf amount is 0, it returns the empty vector.\n\nTime complexity is:\n- effective constant time when `amount` is positive, as the vector structure can be shared\n- linear when `amount` is negative, as the vector needs to be reconstructed.","ref":"Aja.Vector.html#take/2"},{"type":"function","title":"Examples - Aja.Vector.take/2","doc":"iex> Aja.Vector.new(0..100) |> Aja.Vector.take(10)\n    vec([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])\n    iex> Aja.Vector.new([:only_one]) |> Aja.Vector.take(1000)\n    vec([:only_one])\n    iex> Aja.Vector.new(0..10) |> Aja.Vector.take(-5)\n    vec([6, 7, 8, 9, 10])","ref":"Aja.Vector.html#take/2-examples"},{"type":"function","title":"Aja.Vector.take_random/2","doc":"Takes `amount` random elements from `vector`.\n\nNote that, unless `amount` is `0` or `1`, this function will\ntraverse the whole `vector` to get the random sub-vector.\n\nIf `amount` is more than the `vector` size, this is equivalent to shuffling the `vector`:\nthe returned vector cannot be bigger than the original one.\n\nSee `Enum.random/1` for notes on implementation and random seed.\n\nRuns in linear time (except for `amount <= 1`, which is effective constant time).","ref":"Aja.Vector.html#take_random/2"},{"type":"function","title":"Examples - Aja.Vector.take_random/2","doc":"# Although not necessary, let's seed the random algorithm\n    iex> :rand.seed(:exrop, {1, 2, 3})\n    iex> Aja.Vector.new(1..10) |> Aja.Vector.take_random(2)\n    vec([7, 2])\n    iex> Aja.Vector.new([:foo, :bar, :baz]) |> Aja.Vector.take_random(100)\n    vec([:bar, :baz, :foo])","ref":"Aja.Vector.html#take_random/2-examples"},{"type":"function","title":"Aja.Vector.take_while/2","doc":"Takes the elements from the beginning of the `vector` while `fun` returns a truthy value.\n\nRuns in linear time regarding the size of the returned subset.","ref":"Aja.Vector.html#take_while/2"},{"type":"function","title":"Examples - Aja.Vector.take_while/2","doc":"iex> Aja.Vector.new(1..100) |> Aja.Vector.take_while(fn x -> x < 7 end)\n    vec([1, 2, 3, 4, 5, 6])\n    iex> Aja.Vector.new([1, true, %{}, nil, \"abc\"]) |> Aja.Vector.take_while(fn x -> x end)\n    vec([1, true, %{}])","ref":"Aja.Vector.html#take_while/2-examples"},{"type":"function","title":"Aja.Vector.to_list/1","doc":"Converts the `vector` to a list.\n\nRuns in linear time.","ref":"Aja.Vector.html#to_list/1"},{"type":"function","title":"Examples - Aja.Vector.to_list/1","doc":"iex> Aja.Vector.new(10..25) |> Aja.Vector.to_list()\n    [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25]\n    iex> Aja.Vector.new() |> Aja.Vector.to_list()\n    []","ref":"Aja.Vector.html#to_list/1-examples"},{"type":"function","title":"Aja.Vector.uniq/1","doc":"Returns a copy of the vector without any duplicated element.\n\nThe first occurrence of each element is kept.","ref":"Aja.Vector.html#uniq/1"},{"type":"function","title":"Examples - Aja.Vector.uniq/1","doc":"iex> Aja.Vector.new([1, 1, 2, 1, 2, 3, 2]) |> Aja.Vector.uniq()\n    vec([1, 2, 3])","ref":"Aja.Vector.html#uniq/1-examples"},{"type":"function","title":"Aja.Vector.uniq_by/2","doc":"Returns a copy of the vector without elements for which the function `fun` returned duplicate elements.\n\nThe first occurrence of each element is kept.","ref":"Aja.Vector.html#uniq_by/2"},{"type":"function","title":"Examples - Aja.Vector.uniq_by/2","doc":"iex> vector = Aja.Vector.new([x: 1, y: 2, z: 1])\n    vec([x: 1, y: 2, z: 1])\n    iex> Aja.Vector.uniq_by(vector, fn {_x, y} -> y end)\n    vec([x: 1, y: 2])","ref":"Aja.Vector.html#uniq_by/2-examples"},{"type":"function","title":"Aja.Vector.unzip/1","doc":"Opposite of `zip/2`. Extracts two-element tuples from the given `vector` and groups them together.\n\nIt takes a `vector` with elements being two-element tuples and returns a tuple with two vectors,\neach of which is formed by the first and second element of each tuple, respectively.\n\nThis function fails unless `vector` only contains tuples with exactly two elements in each tuple.\n\nRuns in linear time.\n\n    iex> {vector1, vector2} = Aja.Vector.new([{1, :a}, {2, :b}, {3, :c}]) |> Aja.Vector.unzip()\n    iex> vector1\n    vec([1, 2, 3])\n    iex> vector2\n    vec([:a, :b, :c])","ref":"Aja.Vector.html#unzip/1"},{"type":"function","title":"Aja.Vector.update_at/3","doc":"Returns a copy of `vector` with an updated value at the specified `index`.\n\nReturns the `vector` untouched if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#update_at/3"},{"type":"function","title":"Examples - Aja.Vector.update_at/3","doc":"iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at(2, &(&1 * 1000))\n    vec([1, 2, 3000, 4, 5, 6, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at(8, &(&1 * 1000))\n    vec([1, 2, 3, 4, 5, 6, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at(-1, &(&1 * 1000))\n    vec([1, 2, 3, 4, 5, 6, 7, 8000])","ref":"Aja.Vector.html#update_at/3-examples"},{"type":"function","title":"Aja.Vector.update_at!/3","doc":"Returns a copy of `vector` with an updated value at the specified `index`.\n\nRaises an `Aja.Vector.IndexError` if `index` is out of bounds.\nSupports negative indexing from the end of the `vector`.\n\nRuns in effective constant time.","ref":"Aja.Vector.html#update_at!/3"},{"type":"function","title":"Examples - Aja.Vector.update_at!/3","doc":"iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at!(2, &(&1 * 1000))\n    vec([1, 2, 3000, 4, 5, 6, 7, 8])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at!(-1, &(&1 * 1000))\n    vec([1, 2, 3, 4, 5, 6, 7, 8000])\n    iex> Aja.Vector.new(1..8) |> Aja.Vector.update_at!(-9, &(&1 * 1000))\n    ** (Aja.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"Aja.Vector.html#update_at!/3-examples"},{"type":"function","title":"Aja.Vector.with_index/2","doc":"Returns the `vector` with each element wrapped in a tuple alongside its index.\n\nMay receive a function or an integer offset.\n\nIf an integer `offset` is given, it will index from the given `offset` instead of from zero.\n\nIf a `function` is given, it will index by invoking the function for each\nelement and index (zero-based) of the `vector`.\n\nRuns in linear time.","ref":"Aja.Vector.html#with_index/2"},{"type":"function","title":"Examples - Aja.Vector.with_index/2","doc":"iex> vector = Aja.Vector.new([\"foo\", \"bar\", \"baz\"])\n    iex> Aja.Vector.with_index(vector)\n    vec([{\"foo\", 0}, {\"bar\", 1}, {\"baz\", 2}])\n    iex> Aja.Vector.with_index(vector, 100)\n    vec([{\"foo\", 100}, {\"bar\", 101}, {\"baz\", 102}])\n    iex> Aja.Vector.with_index(vector, fn element, index -> {index, element} end)\n    vec([{0, \"foo\"}, {1, \"bar\"}, {2, \"baz\"}])","ref":"Aja.Vector.html#with_index/2-examples"},{"type":"function","title":"Aja.Vector.zip/2","doc":"Zips corresponding elements from two vectors into one vector of tuples.\n\nThe size of the returned vector is the one of the smallest of the input vectors.\n\nRuns in linear time.\n\n    iex> Aja.Vector.zip(Aja.Vector.new([1, 2, 3]), Aja.Vector.new([:a, :b, :c]))\n    vec([{1, :a}, {2, :b}, {3, :c}])\n    iex> Aja.Vector.zip(Aja.Vector.new(0..100), Aja.Vector.new([:a, :b, :c]))\n    vec([{0, :a}, {1, :b}, {2, :c}])","ref":"Aja.Vector.html#zip/2"},{"type":"function","title":"Aja.Vector.zip_with/3","doc":"Zips corresponding elements from two vectors into a new vector,\ntransforming them with the `zip_fun` function as it goes.\n\nThe corresponding elements from each vector are passed to the\nprovided 2-arity `zip_fun` function in turn.\n\nRuns in linear time.\n\n    iex> Aja.Vector.zip_with(Aja.Vector.new([1, 2, 3]), Aja.Vector.new([:a, :b, :c]), &{&2, &1})\n    vec([a: 1, b: 2, c: 3])\n    iex> Aja.Vector.zip_with(Aja.Vector.new(0..100), Aja.Vector.new([:a, :b, :c]), &{&2, &1})\n    vec([a: 0, b: 1, c: 2])","ref":"Aja.Vector.html#zip_with/3"},{"type":"type","title":"Aja.Vector.index/0","doc":"","ref":"Aja.Vector.html#t:index/0"},{"type":"type","title":"Aja.Vector.t/0","doc":"","ref":"Aja.Vector.html#t:t/0"},{"type":"type","title":"Aja.Vector.t/1","doc":"The type of an `Aja.Vector` with elements of the type `value`.\n\nIt should be considered opaque even though it isn't enforced by dialyzer to enable pattern-matching.","ref":"Aja.Vector.html#t:t/1"},{"type":"type","title":"Aja.Vector.value/0","doc":"","ref":"Aja.Vector.html#t:value/0"},{"type":"exception","title":"Aja.Vector.EmptyError","doc":"","ref":"Aja.Vector.EmptyError.html"},{"type":"exception","title":"Aja.Vector.IndexError","doc":"","ref":"Aja.Vector.IndexError.html"},{"type":"extras","title":"README","doc":"[![Hex Version](https://img.shields.io/hexpm/v/aja.svg)](https://hex.pm/packages/aja)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/aja/)\n[![CI](https://github.com/sabiwara/aja/workflows/CI/badge.svg)](https://github.com/sabiwara/aja/actions?query=workflow%3ACI)\n\nExtension of the Elixir standard library focused on data stuctures, data\nmanipulation and performance.\n\n- [Data structures](#data-structures)\n- [Utility functions](#utility-functions)\n- [Installation](#installation)\n- [About Aja](#about-aja)\n- [FAQ](#faq)","ref":"readme.html"},{"type":"extras","title":"Data structures - README","doc":"> \"there is one aspect of functional programming that no amount of cleverness on\n> the part of the compiler writer is likely to mitigate — the use of inferior or\n> inappropriate data structures.\" --\n> [Chris Okasaki](https://www.cs.tufts.edu/~nr/cs257/archive/chris-okasaki/dissertation.pdf)\n\n##","ref":"readme.html#data-structures"},{"type":"extras","title":"Persistent vectors: `Aja.Vector` - README","doc":"A blazing fast, pure Elixir implementation of a persistent vector, meant to\noffer an efficient alternative to lists. Supports many operations like appends\nand random access in effective constant time.\n\n```elixir\niex> vector = Aja.Vector.new(1..10)\nvec([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])\niex> Aja.Vector.append(vector, :foo)\nvec([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, :foo])\niex> vector[3]\n4\niex> Aja.Vector.replace_at(vector, -1, :bar)\nvec([1, 2, 3, 4, 5, 6, 7, 8, 9, :bar])\niex> 3 in vector\ntrue\n```\n\n`Aja.Vector` reimplements many of the functions from the `Enum` module\nspecifically for vectors, with efficiency in mind. It should be easier to use\nfrom Elixir than Erlang's [`:array`](https://erlang.org/doc/man/array.html)\nmodule and faster in most cases.\n\nThe `Aja.vec/1` and `Aja.vec_size/1` macros, while being totally optional, can\nmake it easier to work with vectors and make pattern-matching possible:\n\n```elixir\niex> import Aja\niex> vec([a, 2, c, _d, e]) = Aja.Vector.new(1..5); {a, c, e}\n{1, 3, 5}\niex> vec(first ||| last) = Aja.Vector.new(1..1_000_000); {first, last}\n{1, 1000000}\niex> match?(v when vec_size(v) > 9, vec(1..10))\ntrue\n```\n\nThe `Aja.+++/2` operator provides synctactic sugar for vector concatenation:\n\n```elixir\niex> vec([1, 2, 3]) +++ vec([4, 5])\nvec([1, 2, 3, 4, 5])\n```\n\n##","ref":"readme.html#persistent-vectors-aja-vector"},{"type":"extras","title":"Ordered maps: `Aja.OrdMap` - README","doc":"The standard library does not offer any similar functionality:\n\n- regular maps do not keep track of the insertion order\n- keywords do but they only support atoms and do not have the right performance\n  characteristics (plain lists)\n\n```elixir\niex> %{\"one\" => 1, \"two\" => 2, \"three\" => 3}\n%{\"one\" => 1, \"three\" => 3, \"two\" => 2}\niex> ord_map = Aja.OrdMap.new([{\"one\", 1}, {\"two\", 2}, {\"three\", 3}])\nord(%{\"one\" => 1, \"two\" => 2, \"three\" => 3})\niex> ord_map[\"two\"]\n2\niex> Enum.to_list(ord_map)\n[{\"one\", 1}, {\"two\", 2}, {\"three\", 3}]\n```\n\nOrdered maps behave pretty much like regular maps, and the `Aja.OrdMap` module\noffers the same API as `Map`. The convenience macro `Aja.ord/1` make them a\nbreeze to instantiate or pattern-match upon:\n\n```elixir\niex> import Aja\niex> ord_map = ord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\nord(%{\"一\" => 1, \"二\" => 2, \"三\" => 3})\niex> ord(%{\"三\" => three, \"一\" => one}) = ord_map\niex> {one, three}\n{1, 3}\n```\n\nAll data structures offer:\n\n- great performance characteristics at any size (see [FAQ](#faq))\n- well-documented APIs that are consistent with the standard library\n- implementation of `Inspect`, `Enumerable` and `Collectable` protocols\n- implementation of the `Access` behaviour\n- (optional if `Jason` is installed) implemention of the `Jason.Encoder`\n  protocol\n\n##","ref":"readme.html#ordered-maps-aja-ordmap"},{"type":"extras","title":"Optimized `Enum`: `Aja.Enum` - README","doc":"`Aja.Enum` mirrors the `Enum` module, but its implementation is highly optimized\nfor Aja structures such as `Aja.Vector` or `Aja.OrdMap`.\n\n`Aja.Enum` on vectors/ord maps can often be faster than `Enum` on lists/maps,\ndepending on the function and size of the sequence.","ref":"readme.html#optimized-enum-aja-enum"},{"type":"extras","title":"Installation - README","doc":"Aja can be installed by adding `aja` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:aja, \"~> 0.6.3\"}\n  ]\nend\n```\n\nOr you can just try it out from `iex` or an `.exs` script:\n\n```elixir\niex> Mix.install([:aja])\n:ok\niex> Aja.Vector.new([\"Hello\", \"world!\"])\nvec([\"Hello\", \"world!\"])\n```\n\nDocumentation can be found at [https://hexdocs.pm/aja](https://hexdocs.pm/aja).","ref":"readme.html#installation"},{"type":"extras","title":"About Aja - README","doc":"#","ref":"readme.html#about-aja"},{"type":"extras","title":"Inspirations - README","doc":"- the amazingly polished [Elixir standard library](https://hexdocs.pm/elixir):\n  self-consistent, well-documented and just **delightful** ✨️\n- the also amazing\n  [Python standard library](https://docs.python.org/3/library/), notably its\n  [collections](https://docs.python.org/3/library/collections.html) module\n- various work on efficient\n  [persistent data structures](https://en.wikipedia.org/wiki/Persistent_data_structure)\n  spearheaded by Okasaki\n- Clojure's persistent vectors, by Rich Hickey and influenced by Phil Bagwell\n\n#","ref":"readme.html#inspirations"},{"type":"extras","title":"Goals - README","doc":"- being consistent with Elixir and with itself (API, quality, documentation)\n- no external dependency to help you preserve a decent dependency tree\n- performance-conscious (right algorithm, proper benchmarking, fast compile\n  times\\*)\n- mostly dead-simple pure functions: no configuration, no mandatory macro, no\n  statefulness / OTP\n\n(\\* while fast compile time is a target, vectors are optimized for fast runtime\nat the expense of compile time)\n\n#","ref":"readme.html#goals"},{"type":"extras","title":"Resources - README","doc":"- Chris Okasaki's\n  [Purely Functional Data Structures](https://www.cs.tufts.edu/~nr/cs257/archive/chris-okasaki/dissertation.pdf)\n- Jean Niklas L'orange's\n  [articles](https://hypirion.com/musings/understanding-persistent-vector-pt-1)\n  and [thesis](https://hypirion.com/thesis.pdf) about persistent vectors and RRB\n  trees","ref":"readme.html#resources"},{"type":"extras","title":"FAQ - README","doc":"#","ref":"readme.html#faq"},{"type":"extras","title":"How stable is it? - README","doc":"Aja is still pretty early stage and the high-level organisation is still in\nflux. Expect some breaking changes until it reaches maturity.\n\nHowever, most of its APIs are based on the standard library and should therefore\nremain fairly stable.\n\nBesides, Aja is tested quite thoroughly both with unit tests and property-based\ntesting (especially for data structures). This effort is far from perfect, but\nincreases our confidence in the overall reliability.\n\n#","ref":"readme.html#how-stable-is-it"},{"type":"extras","title":"How is the performance? - README","doc":"##","ref":"readme.html#how-is-the-performance"},{"type":"extras","title":"Vectors - README","doc":"Most operations from `Aja.Vector` are much faster than Erlang's `:array`\nequivalents, and in some cases are even noticeably faster than equivalent list\noperations (map, folds, join, sum...). Make sure to read the efficiency guide\nfrom `Aja.Vector` doc.\n\n##","ref":"readme.html#vectors"},{"type":"extras","title":"Ordered maps - README","doc":"Performance for ordered maps has an inevitable though decent overhead over plain\nmaps in terms of creation and update time (write operations), as well as memory\nusage, since some extra work is needed to keep track of the order. It has\nhowever very good read performance, with a very minimal overhead in terms of key\naccess, and can be enumerated much faster than maps using `Aja.Enum`.\n\n##","ref":"readme.html#ordered-maps"},{"type":"extras","title":"Aja 💖️ JIT - README","doc":"Aja's data structures (vectors and ordered maps) are already pretty fast on\npre-JIT versions of OTP (`<= 23`). Benchmarks on OTP 24 suggest however that\nthey are taking great advantage of the\n[JIT](https://blog.erlang.org/a-first-look-at-the-jit/), relative to lists/maps,\nmaking them even more interesting performance-wise.\n\n##","ref":"readme.html#aja-jit"},{"type":"extras","title":"Benchmarks - README","doc":"Aja data structures should work fine in most cases, but if you're considering\nthem for performance-critical sections of your code, make sure to benchmark\nthem.\n\nBenchmarking is still a work in progress, but you can check the\n[`bench` folder](https://github.com/sabiwara/aja/blob/main/bench) for more\ndetailed figures.","ref":"readme.html#benchmarks"},{"type":"extras","title":"Copyright and License - README","doc":"Aja is licensed under the [MIT License](LICENSE.md).","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Dev - Changelog","doc":"","ref":"changelog.html#dev"},{"type":"extras","title":"v0.6.3 (2024-01-03) - Changelog","doc":"- Stop using an opaque type for `Aja.Vector.t/1:t` to enable pattern-matching.\n  Rely on documentation instead, like `Aja.OrdMap.t/2:t`. Thanks @MegaRedHand!","ref":"changelog.html#v0-6-3-2024-01-03"},{"type":"extras","title":"v0.6.2 (2023-03-10) - Changelog","doc":"#","ref":"changelog.html#v0-6-2-2023-03-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"- `Aja.OrdMap.from_struct/1` respects the defined field order (Elixir >= 1.14)","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.1 (2022-02-04) - Changelog","doc":"#","ref":"changelog.html#v0-6-1-2022-02-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.OrdMap.filter/2` and `Aja.OrdMap.reject/2`\n- Add `Aja.OrdMap.from_keys/2`\n- Bump `ex_doc` to the latest version","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.0 (2021-10-16) - Changelog","doc":"#","ref":"changelog.html#v0-6-0-2021-10-16"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Rename top module from `A` to `Aja`\n- Remove deprecated Aja.ExRange module\n- Remove deprecated functions","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.5.3 (2021-08-20) - Changelog","doc":"#","ref":"changelog.html#v0-5-3-2021-08-20"},{"type":"extras","title":"Enhancement - Changelog","doc":"- Optimize `Aja.OrdMap` internals to reduce memory usage\n\n#","ref":"changelog.html#enhancement"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix compilation warning in deprecated module","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.5.2 (2021-07-01) - Changelog","doc":"#","ref":"changelog.html#v0-5-2-2021-07-01"},{"type":"extras","title":"Enhancement - Changelog","doc":"- Add support for stepped ranges in Elixir 1.12\n- More efficient implementation of vector concatenation\n- Add `Aja.Enum.concat/1` and `Aja.Enum.into/3`\n- Add `Aja.Vector.zip_with/3`\n\n#","ref":"changelog.html#enhancement"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix bug in `Aja.Enum.into/2` when merging into an `Aja.OrdMap`\n- Fix `Aja.vec/1` bug when ranges use negative number\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Deprecate Aja.ExRange","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.5.1 (2021-04-09) - Changelog","doc":"#","ref":"changelog.html#v0-5-1-2021-04-09"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.Vector.flat_map/2`\n- Add `Aja.Vector.reverse/2` and `Aja.Enum.reverse/2`\n- Add `Aja.Enum.concat/2`\n- Add `Aja.Enum.zip/2` and `Aja.Enum.unzip/1`\n- `Aja.OrdMap.drop/2` will only try to period rebuild once after dropping all\n  keys\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix the minimal version needed for `Jason` (1.2)\n- `Aja.OrdMap.take/2` behaves as expected when keys are duplicated\n- `Aja.ord/1` warning on duplicate keys uses the proper stacktrace\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Change `Inspect` protocol for `Aja.Vector` and `Aja.OrdMap`\n- Deprecate Aja.Enum.sort_uniq/1, Aja.Enum.sort_uniq/2","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.5.0 (2021-03-25) - Changelog","doc":"#","ref":"changelog.html#v0-5-0-2021-03-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"- `Aja.OrdMap` new implemention, with highly improved performance ⚡️⚡️\n- `Aja.Enum` as a faster `Enum` module optimized for Aja structures (vectors,\n  ord maps)\n- Added `Aja.ord_size/1` macro\n- `Aja.vec/1` can pattern-match on first and last elements\n- `Aja.ord/1` warns on duplicate errors and can generate the AST on compile time\n  for constant keys\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Reimplement `Aja.OrdMap`, changing its internals completely\n- Change signature of `Aja.OrdMap.foldl/3` and `Aja.OrdMap.foldr/3` for\n  consistency\n- Remove Aja.OrdMap.pop_first/1 and Aja.OrdMap.pop_last/1\n- Modify `Aja.Vector` internals (only breaking if persisted)\n- Deprecate methods from `Aja.Vector` that have been moved to `Aja.Enum`\n- Remove Aja.RBMap and Aja.RBSet\n- Rename `Aja.List.repeat/2` and `Aja.Vector.repeat/2` (previously \"repeatedly\")","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.4.8 (2021-02-23) - Changelog","doc":"#","ref":"changelog.html#v0-4-8-2021-02-23"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.Vector.map_reduce/3`\n- Add `Aja.Vector.scan/2` and `Aja.Vector.scan/3`\n- `Aja.Vector.with_index/2` accepts a function parameter\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Deprecate Aja.RBMap and Aja.RBSet\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix Aja.Vector.join/2 bug when working with chardata","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.7 (2021-02-19) - Changelog","doc":"#","ref":"changelog.html#v0-4-7-2021-02-19"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Vector.reduce/2 and Aja.Vector.reduce/3\n- Add `Aja.Vector.split/2` and `Aja.Vector.split_with/2`\n- Add Aja.Vector.frequencies/1 and Aja.Vector.frequencies_by/2\n- Add Aja.Vector.group_by/3\n- Add `Aja.Vector.dedup/1` and `Aja.Vector.dedup_by/2`\n- Add Aja.Vector.min/2, Aja.Vector.max/2, Aja.Vector.min_by/3,\n  Aja.Vector.max_by/3\n- Improve performance for Aja.Vector.min/1, Aja.Vector.max/1 and\n  `Aja.Vector.uniq/1`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.6 (2021-02-10) - Changelog","doc":"#","ref":"changelog.html#v0-4-6-2021-02-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Vector.find_index/2\n- Add `Aja.Vector.take_while/2`, `Aja.Vector.drop_while/2` and\n  `Aja.Vector.split_while/2`\n- Add `Aja.Vector.zip/2` and `Aja.Vector.unzip/1`\n- Add `Aja.Vector.fetch!/2` alias","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.5 (2021-01-31) - Changelog","doc":"#","ref":"changelog.html#v0-4-5-2021-01-31"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Vector.find/3 and Aja.Vector.find_value/3\n- Add `Aja.+++/2` convenience operator\n- Improve efficiency of vector access functions: `Aja.Vector.at/2`,\n  `Aja.Vector.fetch/2`, `Aja.Vector.replace_at/3`, `Aja.Vector.delete_at/2`...\n  ⚡️\n- Improve compile times\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Invoke callbacks in the right order for: `Aja.Vector.filter/2`,\n  `Aja.Vector.reject/2` and improve performance","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.4 (2021-01-23) - Changelog","doc":"#","ref":"changelog.html#v0-4-4-2021-01-23"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.vec_size/1` macro\n- Add `Aja.Vector.with_index/2`\n- Add Aja.Vector.random/1, `Aja.Vector.take_random/2` and `Aja.Vector.shuffle/1`\n- Drastically improve efficiency of `Aja.Vector.duplicate/2` ⚡️⚡️⚡️\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Changed internal representation of `Aja.Vector` (only breaking if persisted)\n- Stop documenting and exposing internal trees (Aja.RBTree)\n- Rename and deprecate Aja.Vector.append_many/2 to `Aja.Vector.concat/2`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.4.3 (2021-01-12) - Changelog","doc":"This release is mostly focused on vector slicing and performance ⚡️\n\n#","ref":"changelog.html#v0-4-3-2021-01-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Vector.each/2\n- Add `Aja.Vector.slice/2`, `Aja.Vector.slice/3`, `Aja.Vector.take/2` and\n  `Aja.Vector.drop/2`\n- `Aja.Vector` efficiently implements `Enumerable.slice/1`\n- Reimplement `Aja.Vector.delete_at/2`, `Aja.Vector.pop_at/2` efficiently","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.2 (2021-01-10) - Changelog","doc":"#","ref":"changelog.html#v0-4-2-2021-01-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Implement Aja.Vector.product/1\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Invoke callbacks in the right order for: `Aja.Vector.map/2`,\n  `Aja.Vector.map_intersperse/3`, Aja.Vector.map_join/3, Aja.Vector.any?/2,\n  Aja.Vector.all?/2","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.1 (2020-12-05) - Changelog","doc":"#","ref":"changelog.html#v0-4-1-2020-12-05"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Vector.map_join/3 and `Aja.Vector.map_intersperse/3`\n- Implement `Aja.Vector.intersperse/2` more efficiently\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Aja.Vector.sum/1 adds using the same order as `Enum.sum/1`, avoiding slight\n  inconsistencies for floats","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.0 (2020-12-02) - Changelog","doc":"#","ref":"changelog.html#v0-4-0-2020-12-02"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Introduce persistent vectors: `Aja.Vector` 🚀️\n- Add `Aja.vec/1`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.3 (2020-11-14) - Changelog","doc":"#","ref":"changelog.html#v0-3-3-2020-11-14"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.sigil_i/2`\n- Add `Aja.IO.to_iodata/1`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.2 (2020-11-14) - Changelog","doc":"#","ref":"changelog.html#v0-3-2-2020-11-14"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Aja.Enum.sort_uniq/1, Aja.Enum.sort_uniq/2\n- Add `Aja.List.prepend/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.1 (2020-11-05) - Changelog","doc":"#","ref":"changelog.html#v0-3-1-2020-11-05"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Aja.RBMap.Enumerable.member?/2 returns `false` instead of crashing for values\n  other than size-2 tuples","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.3.0 (2020-10-31) - Changelog","doc":"#","ref":"changelog.html#v0-3-0-2020-10-31"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Rework all internals, improved peformance for `Aja.OrdMap`, Aja.RBMap,\n  Aja.RBSet\n- Add `default` parameter to `Aja.OrdMap.first/1`, `Aja.OrdMap.last/1`,\n  Aja.RBMap.first/1, Aja.RBMap.last/1, Aja.RBSet.first/1, Aja.RBSet.last/1\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Changed signature of `Aja.OrdMap.foldl/3`, `Aja.OrdMap.foldr/3`,\n  Aja.RBMap.foldl/3, Aja.RBMap.foldr/3\n- Internals of all data structures have been changed\n- Split Aja.RBTree as Aja.RBTree.Map and Aja.RBTree.Set","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.2.0 (2020-10-25) - Changelog","doc":"#","ref":"changelog.html#v0-2-0-2020-10-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Aja.String.slugify/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Remove `Aja.Array` module","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.1.2 (2020-10-22) - Changelog","doc":"#","ref":"changelog.html#v0-1-2-2020-10-22"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `pop_first/1` and `pop_last/1` to Aja.RBMap, Aja.RBSet and `Aja.OrdMap`\n- Add some guards to functions\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Aja.RBSet.disjoint?/2 was not returning the expected value","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.1.1 (2020-10-21) - Changelog","doc":"#","ref":"changelog.html#v0-1-1-2020-10-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix incompatibility with Elixir 1.10","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.1.0 (2020-10-18) - Changelog","doc":"- Initial release","ref":"changelog.html#v0-1-0-2020-10-18"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2020 Sabiwara\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}